@@ -23,20 +23,25 @@
 #include "string.h"
 #include "gstlame.h"
 
+GST_DEBUG_CATEGORY_STATIC (debug);
+#define GST_CAT_DEFAULT debug
+
 /* elementfactory information */
 static GstElementDetails gst_lame_details = {
   "L.A.M.E. mp3 encoder",
-  "Codec/Audio/Encoder",
+  "Codec/Encoder/Audio",
   "High-quality free MP3 encoder",
   "Erik Walthinsen <omega@cse.ogi.edu>",
 };
 
+/* LAME can do MPEG-1, MPEG-2, and MPEG-2.5, so it has 9 possible
+ * sample rates it supports */
 static GstStaticPadTemplate gst_lame_sink_template =
 GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("audio/x-raw-int, "
-        "endianness = (int) BYTE_ORDER, "
+        "endianness = (int) " G_STRINGIFY (G_BYTE_ORDER) ", "
         "signed = (boolean) true, "
         "width = (int) 16, "
         "depth = (int) 16, "
@@ -69,6 +74,7 @@ gst_lame_mode_get_type (void)
     {4, "4", "Auto"},
     {0, NULL, NULL},
   };
+
   if (!lame_mode_type) {
     lame_mode_type = g_enum_register_static ("GstLameMode", lame_modes);
   }
@@ -93,6 +99,7 @@ gst_lame_quality_get_type (void)
     {9, "9", "9 - Worst"},
     {0, NULL, NULL},
   };
+
   if (!lame_quality_type) {
     lame_quality_type = g_enum_register_static ("GstLameQuality", lame_quality);
   }
@@ -110,6 +117,7 @@ gst_lame_padding_get_type (void)
     {2, "2", "Adjust Padding"},
     {0, NULL, NULL},
   };
+
   if (!lame_padding_type) {
     lame_padding_type = g_enum_register_static ("GstLamePadding", lame_padding);
   }
@@ -156,7 +164,7 @@ enum
   ARG_CWLIMIT,
   ARG_ALLOW_DIFF_SHORT,
   ARG_NO_SHORT_BLOCKS,
-  ARG_EMPHASIS,
+  ARG_EMPHASIS
 };
 
 static void gst_lame_base_init (gpointer g_class);
@@ -338,6 +346,51 @@ gst_lame_class_init (GstLameClass * klass)
   gstelement_class->change_state = gst_lame_change_state;
 }
 
+static GstPadLinkReturn
+gst_lame_src_link (GstPad * pad, const GstCaps * caps)
+{
+  GstLame *lame;
+  gint out_samplerate;
+  GstStructure *structure;
+  GstCaps *othercaps, *channelcaps;
+  GstPadLinkReturn result;
+
+  lame = GST_LAME (gst_pad_get_parent (pad));
+  structure = gst_caps_get_structure (caps, 0);
+
+  if (!gst_structure_get_int (structure, "rate", &out_samplerate) ||
+      !gst_structure_get_int (structure, "channels", &lame->num_channels))
+    g_return_val_if_reached (GST_PAD_LINK_REFUSED);
+
+  if (lame_set_out_samplerate (lame->lgf, out_samplerate) != 0)
+    return GST_PAD_LINK_REFUSED;
+
+  /* we don't do channel conversion */
+  channelcaps = gst_caps_new_simple ("audio/x-raw-int", "channels", G_TYPE_INT,
+      lame->num_channels, NULL);
+  othercaps = gst_caps_intersect (gst_pad_get_pad_template_caps (lame->sinkpad),
+      channelcaps);
+  gst_caps_free (channelcaps);
+
+  result = gst_pad_try_set_caps_nonfixed (lame->sinkpad, othercaps);
+
+  if (GST_PAD_LINK_FAILED (result))
+    return result;
+
+  caps = gst_pad_get_negotiated_caps (lame->sinkpad);
+  structure = gst_caps_get_structure (caps, 0);
+  if (!gst_structure_get_int (structure, "rate", &lame->samplerate))
+    g_return_val_if_reached (GST_PAD_LINK_REFUSED);
+  
+  if (!gst_lame_setup (lame)) {
+    GST_ELEMENT_ERROR (lame, CORE, NEGOTIATION, (NULL),
+        ("could not initialize encoder (wrong parameters?)"));
+    return GST_PAD_LINK_REFUSED;
+  }
+
+  return result;
+}
+
 static GstPadLinkReturn
 gst_lame_sink_link (GstPad * pad, const GstCaps * caps)
 {
@@ -349,8 +402,9 @@ gst_lame_sink_link (GstPad * pad, const GstCaps * caps)
   lame = GST_LAME (gst_pad_get_parent (pad));
   structure = gst_caps_get_structure (caps, 0);
 
-  gst_structure_get_int (structure, "rate", &lame->samplerate);
-  gst_structure_get_int (structure, "channels", &lame->num_channels);
+  if (!gst_structure_get_int (structure, "rate", &lame->samplerate) ||
+      !gst_structure_get_int (structure, "channels", &lame->num_channels))
+    g_return_val_if_reached (GST_PAD_LINK_REFUSED);
 
   if (!gst_lame_setup (lame)) {
     GST_ELEMENT_ERROR (lame, CORE, NEGOTIATION, (NULL),
@@ -385,6 +439,7 @@ gst_lame_init (GstLame * lame)
       gst_pad_new_from_template (gst_static_pad_template_get
       (&gst_lame_src_template), "src");
   gst_element_add_pad (GST_ELEMENT (lame), lame->srcpad);
+  gst_pad_set_link_function (lame->srcpad, gst_lame_src_link);
 
   GST_FLAG_SET (lame, GST_ELEMENT_EVENT_AWARE);
 
@@ -395,9 +450,9 @@ gst_lame_init (GstLame * lame)
   lame->num_channels = 2;
   lame->initialized = FALSE;
 
-  lame->bitrate = lame_get_brate (lame->lgf);
-  lame->compression_ratio = lame_get_compression_ratio (lame->lgf);
-  lame->quality = lame_get_quality (lame->lgf);
+  lame->bitrate = 128;          /* lame_get_brate (lame->lgf); => 0/out of range */
+  lame->compression_ratio = 5;  /* lame_get_compression_ratio (lame->lgf); => 0/out of range */
+  lame->quality = 5;            /* lame_get_quality (lame->lgf); => -1/out of range */
   lame->mode = lame_get_mode (lame->lgf);
   lame->force_ms = lame_get_force_ms (lame->lgf);
   lame->free_format = lame_get_free_format (lame->lgf);
@@ -411,20 +466,20 @@ gst_lame_init (GstLame * lame)
   lame->vbr = lame_get_VBR_q (lame->lgf);
   lame->vbr_mean_bitrate = lame_get_VBR_mean_bitrate_kbps (lame->lgf);
   lame->vbr_min_bitrate = lame_get_VBR_min_bitrate_kbps (lame->lgf);
-  lame->vbr_max_bitrate = lame_get_VBR_max_bitrate_kbps (lame->lgf);
+  lame->vbr_max_bitrate = 320;  /* lame_get_VBR_max_bitrate_kbps (lame->lgf); => 0/no vbr possible */
   lame->vbr_hard_min = lame_get_VBR_hard_min (lame->lgf);
-  lame->lowpass_freq = lame_get_lowpassfreq (lame->lgf);
-  lame->lowpass_width = lame_get_lowpasswidth (lame->lgf);
+  lame->lowpass_freq = 50000;   /* lame_get_lowpassfreq (lame->lgf); => 0/lowpass on everything ? */
+  lame->lowpass_width = 0;      /* lame_get_lowpasswidth (lame->lgf); => -1/out of range */
   lame->highpass_freq = lame_get_highpassfreq (lame->lgf);
-  lame->highpass_width = lame_get_highpasswidth (lame->lgf);
+  lame->highpass_width = 0;     /* lame_get_highpasswidth (lame->lgf); => -1/out of range */
   lame->ath_only = lame_get_ATHonly (lame->lgf);
   lame->ath_short = lame_get_ATHshort (lame->lgf);
   lame->no_ath = lame_get_noATH (lame->lgf);
   /*  lame->ath_type = lame_get_ATHtype (lame->lgf); */
   lame->ath_lower = lame_get_ATHlower (lame->lgf);
-  lame->cwlimit = lame_get_cwlimit (lame->lgf);
+  lame->cwlimit = 8.8717;       /* lame_get_cwlimit (lame->lgf); => 0 */
   lame->allow_diff_short = lame_get_allow_diff_short (lame->lgf);
-  lame->no_short_blocks = lame_get_no_short_blocks (lame->lgf);
+  lame->no_short_blocks = TRUE; /* lame_get_no_short_blocks (lame->lgf); */
   lame->emphasis = lame_get_emphasis (lame->lgf);
   lame->tags = gst_tag_list_new ();
 
@@ -773,7 +828,7 @@ gst_lame_chain (GstPad * pad, GstData * _data)
         } else {
           g_assert_not_reached ();
         }
-        //      gst_pad_event_default (pad, GST_EVENT (buf));
+        gst_pad_event_default (pad, GST_EVENT (buf));
         break;
       default:
         gst_pad_event_default (pad, GST_EVENT (buf));
@@ -794,15 +849,17 @@ gst_lame_chain (GstPad * pad, GstData * _data)
         ((GST_BUFFER_SIZE (buf) / (2 + lame->num_channels)) * 1.25) + 7200;
     mp3_data = g_malloc (mp3_buffer_size);
 
-    if (lame->num_channels == 2) {
-      mp3_size = lame_encode_buffer_interleaved (lame->lgf,
-          (short int *) (GST_BUFFER_DATA (buf)),
-          GST_BUFFER_SIZE (buf) / 4, mp3_data, mp3_buffer_size);
-    } else {
+    /* lame seems to be too stupid to get mono interleaved going */
+    if (lame->num_channels == 1) {
       mp3_size = lame_encode_buffer (lame->lgf,
           (short int *) (GST_BUFFER_DATA (buf)),
           (short int *) (GST_BUFFER_DATA (buf)),
           GST_BUFFER_SIZE (buf) / 2, mp3_data, mp3_buffer_size);
+    } else {
+      mp3_size = lame_encode_buffer_interleaved (lame->lgf,
+          (short int *) (GST_BUFFER_DATA (buf)),
+          GST_BUFFER_SIZE (buf) / 2 / lame->num_channels,
+          mp3_data, mp3_buffer_size);
     }
 
     GST_DEBUG ("encoded %d bytes of audio to %d bytes of mp3",
@@ -853,13 +910,21 @@ gst_lame_chain (GstPad * pad, GstData * _data)
 static gboolean
 gst_lame_setup (GstLame * lame)
 {
+#define CHECK_ERROR(command) G_STMT_START {\
+  if ((command) < 0) { \
+    GST_ERROR_OBJECT (lame, "setup failed: " G_STRINGIFY (command)); \
+    return FALSE; \
+  } \
+}G_STMT_END
   GST_DEBUG_OBJECT (lame, "starting setup");
 
   /* check if we're already initialized; if we are, we might want to check
    * if this initialization is compatible with the previous one */
   /* FIXME: do this */
-  if (lame->initialized)
-    g_warning ("already initialized");
+  if (lame->initialized) {
+    GST_WARNING_OBJECT (lame, "already initialized");
+    lame->initialized = FALSE;
+  }
 
   /* copy the parameters over */
   lame_set_in_samplerate (lame->lgf, lame->samplerate);
@@ -868,43 +933,45 @@ gst_lame_setup (GstLame * lame)
   if (lame->num_channels == 1)
     lame->mode = 3;
 
-  lame_set_brate (lame->lgf, lame->bitrate);
-  lame_set_compression_ratio (lame->lgf, lame->compression_ratio);
-  lame_set_quality (lame->lgf, lame->quality);
-  lame_set_mode (lame->lgf, lame->mode);
-  lame_set_force_ms (lame->lgf, lame->force_ms);
-  lame_set_free_format (lame->lgf, lame->free_format);
-  lame_set_copyright (lame->lgf, lame->copyright);
-  lame_set_original (lame->lgf, lame->original);
-  lame_set_error_protection (lame->lgf, lame->error_protection);
-  lame_set_padding_type (lame->lgf, lame->padding_type);
-  lame_set_extension (lame->lgf, lame->extension);
-  lame_set_strict_ISO (lame->lgf, lame->strict_iso);
-  lame_set_disable_reservoir (lame->lgf, lame->disable_reservoir);
-  lame_set_VBR_q (lame->lgf, lame->vbr);
-  lame_set_VBR_mean_bitrate_kbps (lame->lgf, lame->vbr_mean_bitrate);
-  lame_set_VBR_min_bitrate_kbps (lame->lgf, lame->vbr_min_bitrate);
-  lame_set_VBR_max_bitrate_kbps (lame->lgf, lame->vbr_max_bitrate);
-  lame_set_VBR_hard_min (lame->lgf, lame->vbr_hard_min);
-  lame_set_lowpassfreq (lame->lgf, lame->lowpass_freq);
-  lame_set_lowpasswidth (lame->lgf, lame->lowpass_width);
-  lame_set_highpassfreq (lame->lgf, lame->highpass_freq);
-  lame_set_highpasswidth (lame->lgf, lame->highpass_width);
-  lame_set_ATHonly (lame->lgf, lame->ath_only);
-  lame_set_ATHshort (lame->lgf, lame->ath_short);
-  lame_set_noATH (lame->lgf, lame->no_ath);
-  lame_set_ATHlower (lame->lgf, lame->ath_lower);
-  lame_set_cwlimit (lame->lgf, lame->cwlimit);
-  lame_set_allow_diff_short (lame->lgf, lame->allow_diff_short);
-  lame_set_no_short_blocks (lame->lgf, lame->no_short_blocks);
-  lame_set_emphasis (lame->lgf, lame->emphasis);
+  CHECK_ERROR (lame_set_num_channels (lame->lgf, lame->num_channels));
+  CHECK_ERROR (lame_set_brate (lame->lgf, lame->bitrate));
+  CHECK_ERROR (lame_set_compression_ratio (lame->lgf, lame->compression_ratio));
+  CHECK_ERROR (lame_set_quality (lame->lgf, lame->quality));
+  CHECK_ERROR (lame_set_mode (lame->lgf, lame->mode));
+  CHECK_ERROR (lame_set_force_ms (lame->lgf, lame->force_ms));
+  CHECK_ERROR (lame_set_free_format (lame->lgf, lame->free_format));
+  CHECK_ERROR (lame_set_copyright (lame->lgf, lame->copyright));
+  CHECK_ERROR (lame_set_original (lame->lgf, lame->original));
+  CHECK_ERROR (lame_set_error_protection (lame->lgf, lame->error_protection));
+  CHECK_ERROR (lame_set_padding_type (lame->lgf, lame->padding_type));
+  CHECK_ERROR (lame_set_extension (lame->lgf, lame->extension));
+  CHECK_ERROR (lame_set_strict_ISO (lame->lgf, lame->strict_iso));
+  CHECK_ERROR (lame_set_disable_reservoir (lame->lgf, lame->disable_reservoir));
+  CHECK_ERROR (lame_set_VBR_q (lame->lgf, lame->vbr));
+  CHECK_ERROR (lame_set_VBR_mean_bitrate_kbps (lame->lgf,
+          lame->vbr_mean_bitrate));
+  CHECK_ERROR (lame_set_VBR_min_bitrate_kbps (lame->lgf,
+          lame->vbr_min_bitrate));
+  CHECK_ERROR (lame_set_VBR_max_bitrate_kbps (lame->lgf,
+          lame->vbr_max_bitrate));
+  CHECK_ERROR (lame_set_VBR_hard_min (lame->lgf, lame->vbr_hard_min));
+  CHECK_ERROR (lame_set_lowpassfreq (lame->lgf, lame->lowpass_freq));
+  CHECK_ERROR (lame_set_lowpasswidth (lame->lgf, lame->lowpass_width));
+  CHECK_ERROR (lame_set_highpassfreq (lame->lgf, lame->highpass_freq));
+  CHECK_ERROR (lame_set_highpasswidth (lame->lgf, lame->highpass_width));
+  CHECK_ERROR (lame_set_ATHonly (lame->lgf, lame->ath_only));
+  CHECK_ERROR (lame_set_ATHshort (lame->lgf, lame->ath_short));
+  CHECK_ERROR (lame_set_noATH (lame->lgf, lame->no_ath));
+  CHECK_ERROR (lame_set_ATHlower (lame->lgf, lame->ath_lower));
+  CHECK_ERROR (lame_set_cwlimit (lame->lgf, lame->cwlimit));
+  CHECK_ERROR (lame_set_allow_diff_short (lame->lgf, lame->allow_diff_short));
+  CHECK_ERROR (lame_set_no_short_blocks (lame->lgf, lame->no_short_blocks));
+  CHECK_ERROR (lame_set_emphasis (lame->lgf, lame->emphasis));
 
   gst_lame_set_metadata (lame);
 
   /* initialize the lame encoder */
-  if (lame_init_params (lame->lgf) < 0) {
-    lame->initialized = FALSE;
-  } else {
+  if (lame_init_params (lame->lgf) >= 0) {
     lame->initialized = TRUE;
     /* FIXME: it would be nice to print out the mode here */
     GST_INFO ("lame encoder initialized (%d kbit/s, %d Hz, %d channels)",
@@ -914,6 +981,7 @@ gst_lame_setup (GstLame * lame)
   GST_DEBUG_OBJECT (lame, "done with setup");
 
   return lame->initialized;
+#undef CHECK_ERROR
 }
 
 static GstElementStateReturn
@@ -934,6 +1002,7 @@ gst_lame_change_state (GstElement * element)
     case GST_STATE_READY_TO_NULL:
       if (lame->initialized) {
         lame_close (lame->lgf);
+        lame->lgf = lame_init ();
         lame->initialized = FALSE;
       }
       break;
@@ -954,6 +1023,7 @@ plugin_init (GstPlugin * plugin)
   if (!gst_element_register (plugin, "lame", GST_RANK_NONE, GST_TYPE_LAME))
     return FALSE;
 
+  GST_DEBUG_CATEGORY_INIT (debug, "lame", 0, "lame mp3 encoder");
   return TRUE;
 }
 
