@@ -1,6 +1,5 @@
-/* GStreamer ogg muxer
+/* OGG muxer plugin for GStreamer
  * Copyright (C) 2004 Wim Taymans <wim@fluendo.com>
- * Copyright (C) 2005 Thomas Vander Stichele <thomas at apestaart dot org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -24,7 +23,7 @@
 
 #include <gst/gst.h>
 #include <ogg/ogg.h>
-/* memcpy - if someone knows a way to get rid of it, please speak up
+/* memcpy - if someone knows a way to get rid of it, please speak up 
  * note: the ogg docs even say you need this... */
 #include <string.h>
 #include <time.h>
@@ -38,16 +37,13 @@ GST_DEBUG_CATEGORY_STATIC (gst_ogg_mux_debug);
 #define GST_IS_OGG_MUX(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_OGG_MUX))
 #define GST_IS_OGG_MUX_CLASS(obj) (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_OGG_MUX))
 
-#define GST_GP_FORMAT "[gp %8" G_GUINT64_FORMAT "]"
-
 typedef struct _GstOggMux GstOggMux;
 typedef struct _GstOggMuxClass GstOggMuxClass;
 
 /* all information needed for one ogg stream */
 typedef struct
 {
-  GstPad *pad;                  /* reference for this pad is held by element
-                                 * we belong to */
+  GstPad *pad;                  /* reference for this pad is held by element we belong to */
 
   GstBuffer *buffer;            /* the queued buffer for this pad */
 
@@ -58,18 +54,10 @@ typedef struct
   guint64 duration;             /* duration of current page */
   gboolean eos;
   gint64 offset;
-  GstClockTime timestamp;       /* timestamp for granulepos of last complete
-                                 * packet on this page */
 
   guint state;                  /* state of the pad */
 
   GList *headers;
-
-  GQueue *pagebuffers;          /* list of pages in buffers ready for pushing */
-
-  gboolean new_page;            /* starting a new page */
-  gboolean first_delta;         /* was the first packet in the page a delta */
-  gboolean prev_delta;          /* was the previous buffer a delta frame */
 }
 GstOggPad;
 
@@ -105,10 +93,9 @@ struct _GstOggMux
   guint64 max_delay;
   guint64 max_page_delay;
 
-  GstOggPad *delta_pad;         /* when a delta frame is detected on a stream,
-                                 * we mark pages as delta frames up to the
-                                 * page that has the keyframe */
-
+  gboolean keyframe_mode;       /* when a delta frame is detected on a stream, we mark
+                                   pages as delta frames up to the page that has the
+                                   keyframe */
 };
 
 typedef enum
@@ -136,9 +123,9 @@ enum
   LAST_SIGNAL
 };
 
-/* set to 0.5 seconds by default */
-#define DEFAULT_MAX_DELAY	500000000LL
-#define DEFAULT_MAX_PAGE_DELAY	500000000LL
+/* set to 5 seconds by default */
+#define DEFAULT_MAX_DELAY	5000000000LL
+#define DEFAULT_MAX_PAGE_DELAY	5000000000LL
 enum
 {
   ARG_0,
@@ -155,9 +142,7 @@ static GstStaticPadTemplate src_factory = GST_STATIC_PAD_TEMPLATE ("src",
 static GstStaticPadTemplate sink_factory = GST_STATIC_PAD_TEMPLATE ("sink_%d",
     GST_PAD_SINK,
     GST_PAD_REQUEST,
-    GST_STATIC_CAPS ("video/x-theora; "
-        "audio/x-vorbis; audio/x-flac; audio/x-speex; "
-        "application/x-ogm-video; application/x-ogm-audio")
+    GST_STATIC_CAPS_ANY         /* we can take anything, really */
     );
 
 static void gst_ogg_mux_base_init (gpointer g_class);
@@ -168,12 +153,10 @@ static void gst_ogg_mux_loop (GstElement * element);
 static gboolean gst_ogg_mux_handle_src_event (GstPad * pad, GstEvent * event);
 static GstPad *gst_ogg_mux_request_new_pad (GstElement * element,
     GstPadTemplate * templ, const gchar * name);
-static void gst_ogg_mux_release_request_pad (GstElement * element,
-    GstPad * pad);
-static void gst_ogg_mux_set_property (GObject * object, guint prop_id,
-    const GValue * value, GParamSpec * pspec);
-static void gst_ogg_mux_get_property (GObject * object, guint prop_id,
-    GValue * value, GParamSpec * pspec);
+static void gst_ogg_mux_set_property (GObject * object,
+    guint prop_id, const GValue * value, GParamSpec * pspec);
+static void gst_ogg_mux_get_property (GObject * object,
+    guint prop_id, GValue * value, GParamSpec * pspec);
 static GstElementStateReturn gst_ogg_mux_change_state (GstElement * element);
 
 static GstElementClass *parent_class = NULL;
@@ -230,7 +213,6 @@ gst_ogg_mux_class_init (GstOggMuxClass * klass)
   parent_class = g_type_class_ref (GST_TYPE_ELEMENT);
 
   gstelement_class->request_new_pad = gst_ogg_mux_request_new_pad;
-  gstelement_class->release_pad = gst_ogg_mux_release_request_pad;
 
   g_object_class_install_property (gobject_class, ARG_MAX_DELAY,
       g_param_spec_uint64 ("max-delay", "Max delay",
@@ -252,7 +234,6 @@ gst_ogg_mux_get_sink_event_masks (GstPad * pad)
 {
   static const GstEventMask gst_ogg_mux_sink_event_masks[] = {
     {GST_EVENT_EOS, 0},
-    {GST_EVENT_DISCONTINUOUS, 0},
     {0,}
   };
 
@@ -282,8 +263,6 @@ gst_ogg_mux_init (GstOggMux * ogg_mux)
   ogg_mux->max_delay = DEFAULT_MAX_DELAY;
   ogg_mux->max_page_delay = DEFAULT_MAX_PAGE_DELAY;
 
-  ogg_mux->delta_pad = NULL;
-
   gst_element_set_loop_function (GST_ELEMENT (ogg_mux), gst_ogg_mux_loop);
 }
 
@@ -309,16 +288,18 @@ static void
 gst_ogg_mux_pad_link (GstPad * pad, GstPad * peer, gpointer data)
 {
   GstOggMux *ogg_mux = ogg_mux = GST_OGG_MUX (gst_pad_get_parent (pad));
+  const gchar *padname = gst_pad_get_name (pad);
 
-  GST_DEBUG_OBJECT (ogg_mux, "pad '%s' linked", gst_pad_get_name (pad));
+  GST_DEBUG_OBJECT (ogg_mux, "pad '%s' connected", padname);
 }
 
 static void
 gst_ogg_mux_pad_unlink (GstPad * pad, GstPad * peer, gpointer data)
 {
   GstOggMux *ogg_mux = ogg_mux = GST_OGG_MUX (gst_pad_get_parent (pad));
+  const gchar *padname = gst_pad_get_name (pad);
 
-  GST_DEBUG_OBJECT (ogg_mux, "pad '%s' unlinked", gst_pad_get_name (pad));
+  GST_DEBUG_OBJECT (ogg_mux, "pad '%s' unlinked", padname);
 }
 
 static GstPad *
@@ -369,10 +350,6 @@ gst_ogg_mux_request_new_pad (GstElement * element,
       oggpad->eos = FALSE;
       /* we assume there will be some control data first for this pad */
       oggpad->state = GST_OGG_PAD_STATE_CONTROL;
-      oggpad->new_page = TRUE;
-      oggpad->first_delta = FALSE;
-      oggpad->prev_delta = FALSE;
-      oggpad->pagebuffers = g_queue_new ();
 
       /* save a pointer to our data in the pad */
       gst_pad_set_element_private (newpad, oggpad);
@@ -398,31 +375,6 @@ gst_ogg_mux_request_new_pad (GstElement * element,
   return newpad;
 }
 
-/* FIXME: I haven't seen this function actually get called with gst-launch */
-static void
-gst_ogg_mux_release_request_pad (GstElement * element, GstPad * pad)
-{
-  GSList *walk;
-  GstOggPad *oggpad = NULL;
-  GstOggMux *mux = GST_OGG_MUX (element);
-
-  walk = mux->sinkpads;
-  while (walk) {
-    oggpad = (GstOggPad *) walk->data;
-    if (oggpad->pad == pad) {
-      GstBuffer *buf;
-
-      while ((buf = GST_BUFFER (g_queue_pop_head (oggpad->pagebuffers)))) {
-        gst_buffer_unref (buf);
-      }
-      g_queue_free (oggpad->pagebuffers);
-      oggpad->pagebuffers = NULL;
-      break;
-    }
-    walk = walk->next;
-  }
-}
-
 /* handle events */
 static gboolean
 gst_ogg_mux_handle_src_event (GstPad * pad, GstEvent * event)
@@ -446,13 +398,13 @@ gst_ogg_mux_handle_src_event (GstPad * pad, GstEvent * event)
 }
 
 static GstBuffer *
-gst_ogg_mux_next_buffer (GstOggPad * pad, gboolean * interrupt)
+gst_ogg_mux_next_buffer (GstOggPad * pad)
 {
   GstData *data = NULL;
-  GstBuffer *buffer = NULL;
 
   while (data == NULL) {
     data = gst_pad_pull (pad->pad);
+    GST_DEBUG ("muxer: pulled %s:%s %p", GST_DEBUG_PAD_NAME (pad->pad), data);
     /* if it's an event, handle it */
     if (GST_IS_EVENT (data)) {
       GstEventType type;
@@ -464,20 +416,12 @@ gst_ogg_mux_next_buffer (GstOggPad * pad, gboolean * interrupt)
 
       switch (type) {
         case GST_EVENT_EOS:
-          pad->eos = TRUE;
           gst_event_unref (event);
           return NULL;
-        case GST_EVENT_INTERRUPT:
-          *interrupt = TRUE;
-          return NULL;
         case GST_EVENT_DISCONTINUOUS:
         {
-          gint64 value;
+          gint64 value = 0;
 
-          if (GST_EVENT_DISCONT_NEW_MEDIA (event)) {
-            gst_pad_event_default (pad->pad, event);
-            break;
-          }
           if (gst_event_discont_get_value (event, GST_FORMAT_TIME, &value)) {
             GST_DEBUG_OBJECT (ogg_mux,
                 "got discont of %" G_GUINT64_FORMAT " on pad %s:%s",
@@ -485,9 +429,8 @@ gst_ogg_mux_next_buffer (GstOggPad * pad, gboolean * interrupt)
           }
           pad->offset = value;
           gst_event_unref (event);
-
-        }
           break;
+        }
         default:
           gst_pad_event_default (pad->pad, event);
           break;
@@ -502,34 +445,24 @@ gst_ogg_mux_next_buffer (GstOggPad * pad, gboolean * interrupt)
       if (pad->state == GST_OGG_PAD_STATE_CONTROL) {
         /* and we have one */
         if (incaps) {
-          GST_DEBUG_OBJECT (pad->pad,
-              "got incaps buffer in control state, ignoring");
+          GST_DEBUG ("muxer: got incaps buffer in control state, ignoring");
           /* just ignore */
           gst_buffer_unref (buf);
-          /* pull next one in next iteration */
           data = NULL;
         } else {
-          GST_DEBUG_OBJECT (pad->pad,
-              "got data buffer in control state, switching to data mode");
+          GST_DEBUG
+              ("muxer: got data buffer in control state, switching to data mode");
           /* this is a data buffer so switch to data state */
           pad->state = GST_OGG_PAD_STATE_DATA;
         }
       }
     }
   }
-  buffer = GST_BUFFER (data);
-  GST_DEBUG_OBJECT (pad->pad,
-      GST_GP_FORMAT " got next GstBuffer (time %" GST_TIME_FORMAT ")",
-      GST_BUFFER_OFFSET_END (buffer),
-      GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buffer)));
-  return buffer;
+  return GST_BUFFER (data);
 }
 
-/* create a GstBuffer from the ogg page
- * use OFFSET_END temporarily to store the page's granulepos,
- * caller is responsible for setting OFFSET + SIZE later */
 static GstBuffer *
-gst_ogg_mux_buffer_from_page (GstOggMux * mux, ogg_page * page, gboolean delta)
+gst_ogg_mux_buffer_from_page (GstOggMux * mux, ogg_page * page)
 {
   GstBuffer *buffer;
 
@@ -544,144 +477,31 @@ gst_ogg_mux_buffer_from_page (GstOggMux * mux, ogg_page * page, gboolean delta)
   GST_BUFFER_TIMESTAMP (buffer) = mux->next_ts;
   GST_BUFFER_OFFSET (buffer) = mux->offset;
   mux->offset += GST_BUFFER_SIZE (buffer);
-  /* here we set granulepos as OFFSET_END to use for debugging;
-   * before we push we should set it to OFFSET + SIZE */
-  GST_BUFFER_OFFSET_END (buffer) = ogg_page_granulepos (page);
-  if (delta)
-    GST_BUFFER_FLAG_SET (buffer, GST_BUFFER_DELTA_UNIT);
+  GST_BUFFER_OFFSET_END (buffer) = mux->offset;
 
   return buffer;
 }
 
 static void
-gst_ogg_mux_push_buffer (GstOggMux * mux, GstBuffer * buffer)
+gst_ogg_mux_push_page (GstOggMux * mux, ogg_page * page)
 {
   if (GST_PAD_IS_USABLE (mux->srcpad)) {
-    GST_BUFFER_OFFSET_END (buffer) =
-        GST_BUFFER_OFFSET (buffer) + GST_BUFFER_SIZE (buffer);
+    GstBuffer *buffer = gst_ogg_mux_buffer_from_page (mux, page);
+
     gst_pad_push (mux->srcpad, GST_DATA (buffer));
   }
 }
 
-/* if all queues have at least one page, dequeue the page with the lowest
- * timestamp */
-static gboolean
-gst_ogg_mux_dequeue_page (GstOggMux * mux)
-{
-  GSList *walk;
-  GstOggPad *opad = NULL;       /* "oldest" pad */
-  GstClockTime oldest = GST_CLOCK_TIME_NONE;
-  GstBuffer *buf = NULL;
-  gboolean ret = FALSE;
-
-  walk = mux->sinkpads;
-  while (walk) {
-    GstOggPad *pad = (GstOggPad *) walk->data;
-
-    /* We need each queue to either be at EOS, or have one or more pages
-     * available with a set granulepos (i.e. not -1), otherwise we don't have
-     * enough data yet to determine which stream needs to go next for correct
-     * time ordering. */
-    if (pad->pagebuffers->length == 0) {
-      if (pad->eos) {
-        GST_LOG_OBJECT (pad->pad, "pad is EOS, skipping for dequeue decision");
-      } else {
-        GST_LOG_OBJECT (pad->pad, "no pages in this queue, can't dequeue");
-        return FALSE;
-      }
-    } else {
-      /* We then need to check for a non-negative granulepos */
-      int i;
-      gboolean valid = FALSE;
-
-      for (i = 0; i < pad->pagebuffers->length; i++) {
-        buf = g_queue_peek_nth (pad->pagebuffers, i);
-        if (GST_BUFFER_OFFSET_END (buf) != -1) {
-          valid = TRUE;
-          break;
-        }
-      }
-      if (!valid) {
-        GST_LOG_OBJECT (pad->pad, "No page timestamps in queue, can't dequeue");
-        return FALSE;
-      }
-    }
-    walk = walk->next;
-  }
-
-  walk = mux->sinkpads;
-  while (walk) {
-    GstOggPad *pad = (GstOggPad *) walk->data;
-
-    /* any page with a granulepos of -1 can be pushed immediately */
-    buf = g_queue_peek_head (pad->pagebuffers);
-    while (buf && GST_BUFFER_OFFSET_END (buf) == -1) {
-      GST_LOG_OBJECT (pad->pad, GST_GP_FORMAT " pushing page", -1);
-      g_queue_pop_head (pad->pagebuffers);
-      gst_ogg_mux_push_buffer (mux, buf);
-      buf = g_queue_peek_head (pad->pagebuffers);
-      ret = TRUE;
-    }
-
-    if (buf) {
-      /* if no oldest buffer yet, take this one */
-      if (oldest == GST_CLOCK_TIME_NONE) {
-        oldest = GST_BUFFER_TIMESTAMP (buf);
-        opad = pad;
-      } else {
-        /* if we have an oldest, compare with this one */
-        if (GST_BUFFER_TIMESTAMP (buf) < oldest) {
-          oldest = GST_BUFFER_TIMESTAMP (buf);
-          opad = pad;
-        }
-      }
-    }
-    walk = walk->next;
-  }
-
-  if (oldest != GST_CLOCK_TIME_NONE) {
-    g_assert (opad);
-    buf = g_queue_pop_head (opad->pagebuffers);
-    GST_LOG_OBJECT (opad->pad,
-        GST_GP_FORMAT " pushing oldest page (time %" GST_TIME_FORMAT ")",
-        GST_BUFFER_OFFSET_END (buf),
-        GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buf)));
-    gst_ogg_mux_push_buffer (mux, buf);
-    ret = TRUE;
-  }
-
-  return ret;
-}
-
-/* put the given page on a per-pad queue, timestamping it correctly.
- * after that, dequeue and push as many pages as possible
- * before calling me, make sure that the the pad's timestamp matches
- * the page's granulepos */
-static void
-gst_ogg_mux_pad_queue_page (GstOggMux * mux, GstOggPad * pad, ogg_page * page,
-    gboolean delta)
-{
-  GstBuffer *buffer = gst_ogg_mux_buffer_from_page (mux, page, delta);
-
-  /* take the timestamp of the last completed packet on this page */
-  GST_BUFFER_TIMESTAMP (buffer) = pad->timestamp;
-  g_queue_push_tail (pad->pagebuffers, buffer);
-  GST_LOG_OBJECT (pad->pad, GST_GP_FORMAT " queued buffer page (time %" GST_TIME_FORMAT "), %d page buffers queued", ogg_page_granulepos (page), GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buffer)), pad->pagebuffers->length);      /* no g_queue_get_length in 2.2 */
-
-  while (gst_ogg_mux_dequeue_page (mux));
-}
-
-
 /*
  * Given two pads, compare the buffers queued on it and return 0 if they have
- * an equal priority, 1 if the new pad is better, -1 if the old pad is better
+ * an equal priority, 1 if the new pad is better, -1 if the old pad is better 
  */
 static gint
 gst_ogg_mux_compare_pads (GstOggMux * ogg_mux, GstOggPad * old, GstOggPad * new)
 {
   guint64 oldtime, newtime;
 
-  /* if the old pad doesn't contain anything or is even NULL, return
+  /* if the old pad doesn't contain anything or is even NULL, return 
    * the new pad as best candidate and vice versa */
   if (old == NULL || old->buffer == NULL)
     return 1;
@@ -718,10 +538,9 @@ gst_ogg_mux_compare_pads (GstOggMux * ogg_mux, GstOggPad * old, GstOggPad * new)
 }
 
 /* make sure a buffer is queued on all pads, returns a pointer to an oggpad
- * that holds the "best"  buffer or NULL when no pad was usable
- * "best" means the buffer marked with the lowest timestamp */
+ * that holds the best buffer or NULL when no pad was usable */
 static GstOggPad *
-gst_ogg_mux_queue_pads (GstOggMux * ogg_mux, gboolean * interrupt)
+gst_ogg_mux_queue_pads (GstOggMux * ogg_mux)
 {
   GstOggPad *bestpad = NULL;
   GSList *walk;
@@ -733,14 +552,9 @@ gst_ogg_mux_queue_pads (GstOggMux * ogg_mux, gboolean * interrupt)
 
     walk = walk->next;
 
-    if (pad->eos)
-      continue;
-
     /* try to get a new buffer for this pad if needed and possible */
     if (pad->buffer == NULL && GST_PAD_IS_USABLE (pad->pad)) {
-      pad->buffer = gst_ogg_mux_next_buffer (pad, interrupt);
-      if (*interrupt)
-        return NULL;
+      pad->buffer = gst_ogg_mux_next_buffer (pad);
       /* no next buffer, try another pad */
       if (pad->buffer == NULL)
         continue;
@@ -771,8 +585,7 @@ gst_ogg_mux_get_headers (GstOggPad * pad)
 
   ogg_mux = GST_OGG_MUX (gst_pad_get_parent (pad->pad));
 
-  GST_LOG_OBJECT (ogg_mux, "getting headers from pad %s:%s",
-      GST_DEBUG_PAD_NAME (pad->pad));
+  GST_LOG ("getting headers from pad %s:%s", GST_DEBUG_PAD_NAME (pad->pad));
 
   caps = gst_pad_get_negotiated_caps (pad->pad);
   if (caps != NULL) {
@@ -834,7 +647,7 @@ gst_ogg_mux_set_header_on_caps (GstCaps * caps, GList * buffers)
  * page that allows decoders to identify the type of the stream.
  * After that we need to write out all extra info for the decoders.
  * In the case of a codec that also needs data as configuration, we can
- * find that info in the streamcaps.
+ * find that info in the streamcaps. 
  * After writing the headers we must start a new page for the data.
  */
 static void
@@ -846,7 +659,7 @@ gst_ogg_mux_send_headers (GstOggMux * mux)
 
   hbufs = NULL;
 
-  GST_LOG_OBJECT (mux, "collecting headers");
+  GST_LOG ("collecting headers");
 
   walk = mux->sinkpads;
   while (walk) {
@@ -854,7 +667,7 @@ gst_ogg_mux_send_headers (GstOggMux * mux)
 
     walk = walk->next;
 
-    GST_LOG_OBJECT (mux, "looking at pad %s:%s", GST_DEBUG_PAD_NAME (pad->pad));
+    GST_LOG ("looking at pad %s:%s", GST_DEBUG_PAD_NAME (pad->pad));
 
     /* if the pad has no buffer, we don't care */
     if (pad->buffer == NULL)
@@ -864,7 +677,7 @@ gst_ogg_mux_send_headers (GstOggMux * mux)
     pad->headers = gst_ogg_mux_get_headers (pad);
   }
 
-  GST_LOG_OBJECT (mux, "creating first headers");
+  GST_LOG ("creating first headers");
   walk = mux->sinkpads;
   while (walk) {
     GstOggPad *pad = (GstOggPad *) walk->data;
@@ -876,23 +689,21 @@ gst_ogg_mux_send_headers (GstOggMux * mux)
 
     pad->packetno = 0;
 
-    GST_LOG_OBJECT (mux, "looping over headers for pad %s:%s",
+    GST_LOG ("looping over headers for pad %s:%s",
         GST_DEBUG_PAD_NAME (pad->pad));
 
     if (pad->headers) {
       buf = GST_BUFFER (pad->headers->data);
       pad->headers = g_list_remove (pad->headers, buf);
     } else {
-      buf = pad->buffer;
-      gst_buffer_ref (buf);
+      /* create empty buffer for the stream */
+      buf = gst_buffer_new_and_alloc (0);
     }
 
     /* create a packet from the buffer */
     packet.packet = GST_BUFFER_DATA (buf);
     packet.bytes = GST_BUFFER_SIZE (buf);
     packet.granulepos = GST_BUFFER_OFFSET_END (buf);
-    if (packet.granulepos == -1)
-      packet.granulepos = 0;
     /* mark BOS and packet number */
     packet.b_o_s = (pad->packetno == 0);
     packet.packetno = pad->packetno++;
@@ -903,23 +714,23 @@ gst_ogg_mux_send_headers (GstOggMux * mux)
     ogg_stream_packetin (&pad->stream, &packet);
     gst_buffer_unref (buf);
 
-    GST_LOG_OBJECT (mux, "flushing page with first packet");
+    GST_LOG ("flushing page with first packet");
     while (ogg_stream_flush (&pad->stream, &page)) {
-      GstBuffer *hbuf = gst_ogg_mux_buffer_from_page (mux, &page, FALSE);
+      GstBuffer *hbuf = gst_ogg_mux_buffer_from_page (mux, &page);
 
-      GST_LOG_OBJECT (mux, "swapped out page");
+      GST_LOG ("swapped out page");
       hbufs = g_list_append (hbufs, hbuf);
     }
   }
 
-  GST_LOG_OBJECT (mux, "creating next headers");
+  GST_LOG ("creating next headers");
   walk = mux->sinkpads;
   while (walk) {
     GstOggPad *pad = (GstOggPad *) walk->data;
 
     walk = walk->next;
 
-    GST_LOG_OBJECT (mux, "looping over headers for pad %s:%s",
+    GST_LOG ("looping over headers for pad %s:%s",
         GST_DEBUG_PAD_NAME (pad->pad));
 
     hwalk = pad->headers;
@@ -934,8 +745,6 @@ gst_ogg_mux_send_headers (GstOggMux * mux)
       packet.packet = GST_BUFFER_DATA (buf);
       packet.bytes = GST_BUFFER_SIZE (buf);
       packet.granulepos = GST_BUFFER_OFFSET_END (buf);
-      if (packet.granulepos == -1)
-        packet.granulepos = 0;
       /* mark BOS and packet number */
       packet.b_o_s = (pad->packetno == 0);
       packet.packetno = pad->packetno++;
@@ -948,22 +757,21 @@ gst_ogg_mux_send_headers (GstOggMux * mux)
 
       /* if last header, flush page */
       if (hwalk == NULL) {
-        GST_LOG_OBJECT (mux,
-            "flushing page as packet %" G_GINT64_FORMAT
-            " is first or last packet", pad->packetno);
+        GST_LOG ("flushing page as packet %d is first or last packet",
+            pad->packetno);
         while (ogg_stream_flush (&pad->stream, &page)) {
-          GstBuffer *hbuf = gst_ogg_mux_buffer_from_page (mux, &page, FALSE);
+          GstBuffer *hbuf = gst_ogg_mux_buffer_from_page (mux, &page);
 
-          GST_LOG_OBJECT (mux, "swapped out page");
+          GST_LOG ("swapped out page");
           hbufs = g_list_append (hbufs, hbuf);
         }
       } else {
-        GST_LOG_OBJECT (mux, "try to swap out page");
+        GST_LOG ("try to swap out page");
         /* just try to swap out a page then */
         while (ogg_stream_pageout (&pad->stream, &page) > 0) {
-          GstBuffer *hbuf = gst_ogg_mux_buffer_from_page (mux, &page, FALSE);
+          GstBuffer *hbuf = gst_ogg_mux_buffer_from_page (mux, &page);
 
-          GST_LOG_OBJECT (mux, "swapped out page");
+          GST_LOG ("swapped out page");
           hbufs = g_list_append (hbufs, hbuf);
         }
       }
@@ -998,67 +806,44 @@ gst_ogg_mux_send_headers (GstOggMux * mux)
 /* basic idea:
  *
  * 1) find a pad to pull on, this is done by pulling on all pads and
- *    looking at the buffers, taking the "oldest" one first.
- * 2) store the selected pad and keep on pulling until we fill a
- *    complete ogg page or the ogg page is filled above the max-delay
+ *    looking at the buffers to decide which one should be muxed first.
+ * 2) store the selected pad and keep on pulling until we fill a 
+ *    complete ogg page or the ogg page is filled above the max-delay 
  *    threshold. This is needed because the ogg spec says that
  *    you should fill a complete page with data from the same logical
  *    stream. When the page is filled, go back to 1).
- * 3) before filling a page, read ahead one more buffer to see if this
+ * 3) before filling a packet, read ahead one more buffer to see if this
  *    packet is the last of the stream. We need to do this because the ogg
  *    spec mandates that the last packet should have the EOS flag set before
  *    sending it to ogg.
- *    FIXME: this is not necessarily true; Ogg spec says you are allowed to
- *    send "nil" eos pages, which might be a nice way to do this (thomasvs)
- * 4) pages get queued on a per-pad queue.  Every time a page gets queued,
- *    a dequeue is called, which will dequeue the oldest page on any pad,
- *    provided that ALL pads have at least one marked page in the queue.
  */
 static void
 gst_ogg_mux_loop (GstElement * element)
 {
   GstOggMux *ogg_mux;
   GstOggPad *best;
-  gboolean delta_unit;
-  gboolean interrupt = FALSE;
-  GstClockTime timestamp;       /* timestamp of the packetin'd buffer */
-  gint64 granulepos = 0;        /* granulepos of the packetin'd buffer */
 
   ogg_mux = GST_OGG_MUX (element);
 
-  /* queue buffers on all pads; find a buffer with the lowest timestamp */
-  best = gst_ogg_mux_queue_pads (ogg_mux, &interrupt);
-  if (interrupt)
-    return;
+  best = gst_ogg_mux_queue_pads (ogg_mux);
 
-  /* if we were already pulling from one pad, but the new "best" buffer is
-   * from another pad, we need to check if we have reason to flush a page
-   * for the pad we were pulling from before */
+  /* we're pulling a pad and there is a better one, see if we need
+   * to flush the current page */
   if (ogg_mux->pulling && best &&
       ogg_mux->pulling != best && ogg_mux->pulling->buffer) {
-    GstOggPad *pad = ogg_mux->pulling;
-
     GstClockTime last_ts =
-        GST_BUFFER_TIMESTAMP (pad->buffer) + GST_BUFFER_DURATION (pad->buffer);
-    /* if the next packet in the current page is going to make the page
+        GST_BUFFER_TIMESTAMP (ogg_mux->pulling->buffer) +
+        GST_BUFFER_DURATION (ogg_mux->pulling->buffer);
+    /* if the next packet in the current page is going to make the page 
      * too long, we need to flush */
     if (last_ts > ogg_mux->next_ts + ogg_mux->max_delay) {
       ogg_page page;
 
-      GST_LOG_OBJECT (pad->pad,
-          GST_GP_FORMAT " stored packet %" G_GINT64_FORMAT
-          " will make page too long, flushing",
-          GST_BUFFER_OFFSET_END (pad->buffer), pad->stream.packetno);
-
-      while (ogg_stream_flush (&pad->stream, &page)) {
-        /* pages get put on a per-pad queue */
-        gst_ogg_mux_pad_queue_page (ogg_mux, pad, &page, pad->first_delta);
+      while (ogg_stream_flush (&ogg_mux->pulling->stream, &page)) {
+        gst_ogg_mux_push_page (ogg_mux, &page);
         /* increment the page number counter */
-        pad->pageno++;
-        /* mark other pages as delta */
-        pad->first_delta = TRUE;
+        ogg_mux->pulling->pageno++;
       }
-      pad->new_page = TRUE;
       ogg_mux->pulling = NULL;
     }
   }
@@ -1078,29 +863,25 @@ gst_ogg_mux_loop (GstElement * element)
       return;
     }
   }
-
   if (ogg_mux->need_headers) {
     gst_ogg_mux_send_headers (ogg_mux);
     ogg_mux->need_headers = FALSE;
   }
 
   /* we are pulling from a pad, continue to do so until a page
-   * has been filled and queued */
+   * has been filled and pushed */
   if (ogg_mux->pulling != NULL) {
     ogg_packet packet;
     ogg_page page;
     GstBuffer *buf, *tmpbuf;
     GstOggPad *pad = ogg_mux->pulling;
     gint64 duration;
-    gboolean force_flush;
 
     /* now see if we have a buffer */
     buf = pad->buffer;
     if (buf == NULL) {
       /* no buffer, get one, and store in the pad so we free it later on */
-      buf = pad->buffer = gst_ogg_mux_next_buffer (pad, &interrupt);
-      if (interrupt)
-        return;
+      buf = pad->buffer = gst_ogg_mux_next_buffer (pad);
       /* data exhausted on this pad (EOS) */
       if (buf == NULL) {
         /* stop pulling from the pad */
@@ -1109,211 +890,61 @@ gst_ogg_mux_loop (GstElement * element)
       }
     }
 
-    delta_unit = GST_BUFFER_FLAG_IS_SET (buf, GST_BUFFER_DELTA_UNIT);
     duration = GST_BUFFER_DURATION (buf);
 
     /* create a packet from the buffer */
     packet.packet = GST_BUFFER_DATA (buf);
     packet.bytes = GST_BUFFER_SIZE (buf);
     packet.granulepos = GST_BUFFER_OFFSET_END (buf);
-    if (packet.granulepos == -1)
-      packet.granulepos = 0;
     /* mark BOS and packet number */
     packet.b_o_s = (pad->packetno == 0);
     packet.packetno = pad->packetno++;
-    GST_DEBUG_OBJECT (pad->pad, GST_GP_FORMAT
-        " packet %" G_GINT64_FORMAT " (%ld bytes) created from buffer",
-        packet.granulepos, packet.packetno, packet.bytes);
 
     /* read ahead one more buffer to find EOS */
-    tmpbuf = gst_ogg_mux_next_buffer (pad, &interrupt);
-    if (interrupt)
-      return;
+    tmpbuf = gst_ogg_mux_next_buffer (pad);
     /* data exhausted on this pad */
     if (tmpbuf == NULL) {
       /* stop pulling from the pad */
       ogg_mux->pulling = NULL;
     }
     /* mark EOS */
-    if (tmpbuf == NULL) {
-      GST_DEBUG_OBJECT (pad->pad,
-          GST_GP_FORMAT " packet is last in stream, setting EOS",
-          packet.granulepos);
-      packet.e_o_s = 1;
-    } else {
-      packet.e_o_s = 0;
-    }
-
-    /* we flush the currently built up page if this new packet has a keyframe */
-    force_flush = (pad->prev_delta && !delta_unit);
-    if (duration != -1) {
-      pad->duration += duration;
-      /* if page duration exceeds max, flush page */
-      if (pad->duration > ogg_mux->max_page_delay) {
-        force_flush = TRUE;
-        pad->duration = 0;
-      }
-    }
-
-    /* flush out the currently built page if necessary */
-    if (force_flush) {
-      GST_LOG_OBJECT (pad->pad,
-          GST_GP_FORMAT " forcing flush because of keyframe",
-          GST_BUFFER_OFFSET_END (pad->buffer));
-      while (ogg_stream_flush (&pad->stream, &page)) {
-        gst_ogg_mux_pad_queue_page (ogg_mux, pad, &page, pad->first_delta);
-
-        /* increment the page number counter */
-        pad->pageno++;
-        /* mark other pages as delta */
-        pad->first_delta = TRUE;
-      }
-      pad->new_page = TRUE;
-    }
-
-    /* if this is the first packet of a new page figure out the delta flag */
-    if (pad->new_page) {
-      if (delta_unit) {
-        /* This page is a delta frame */
-        if (ogg_mux->delta_pad == NULL) {
-          /* we got a delta unit on this pad */
-          ogg_mux->delta_pad = pad;
-        }
-        /* mark the page as delta */
-        pad->first_delta = TRUE;
-      } else {
-        /* got a keyframe */
-        if (ogg_mux->delta_pad == pad) {
-          /* if we get it on the pad with deltaunits,
-           * we mark the page as non delta */
-          pad->first_delta = FALSE;
-        } else if (ogg_mux->delta_pad != NULL) {
-          /* if there are pads with delta frames, we
-           * must mark this one as delta */
-          pad->first_delta = TRUE;
-        } else {
-          pad->first_delta = FALSE;
-        }
-      }
-      pad->new_page = FALSE;
-    }
-
-    /* save key unit to track delta->key unit transitions */
-    pad->prev_delta = delta_unit;
+    packet.e_o_s = (tmpbuf == NULL ? 1 : 0);
 
     /* swap the packet in */
-    if (packet.e_o_s == 1)
-      GST_DEBUG_OBJECT (pad, "swapping in EOS packet");
-    if (packet.b_o_s == 1)
-      GST_DEBUG_OBJECT (pad, "swapping in BOS packet");
-
     ogg_stream_packetin (&pad->stream, &packet);
 
-    timestamp = GST_BUFFER_TIMESTAMP (pad->buffer);
-    granulepos = GST_BUFFER_OFFSET_END (pad->buffer);
-
-    GST_LOG_OBJECT (pad->pad,
-        GST_GP_FORMAT " packet %" G_GINT64_FORMAT ", time %"
-        GST_TIME_FORMAT ") packetin'd",
-        granulepos, packet.packetno, GST_TIME_ARGS (timestamp));
     /* don't need the old buffer anymore */
     gst_buffer_unref (pad->buffer);
     /* store new readahead buffer */
     pad->buffer = tmpbuf;
 
-    /* let ogg write out the pages now. The packet we got could end
-     * up in more than one page so we need to write them all */
-    if (ogg_stream_pageout (&pad->stream, &page) > 0) {
-      /* we have a new page, so we need to timestamp it correctly.
-       * if this fresh packet ends on this page, then the page's granulepos
-       * comes from that packet, and we should set this buffer's timestamp */
-
-      GST_LOG_OBJECT (pad->pad,
-          GST_GP_FORMAT " packet %" G_GINT64_FORMAT ", time %"
-          GST_TIME_FORMAT ") caused new page",
-          granulepos, packet.packetno, GST_TIME_ARGS (timestamp));
-      GST_LOG_OBJECT (pad->pad,
-          GST_GP_FORMAT " new page %d", ogg_page_granulepos (&page),
-          pad->stream.pageno);
-
-      if (ogg_page_granulepos (&page) == granulepos) {
-        /* the  packet we streamed in finishes on the page,
-         * because the page's granulepos is the granulepos of the last
-         * packet completed on that page,
-         * so update the timestamp that we will give to the page */
-        GST_LOG_OBJECT (pad->pad,
-            GST_GP_FORMAT
-            " packet finishes on new page, updating timestamp to %"
-            GST_TIME_FORMAT, granulepos, GST_TIME_ARGS (timestamp));
-        pad->timestamp = timestamp;
-      } else {
-        GST_LOG_OBJECT (pad->pad,
-            GST_GP_FORMAT
-            " packet spans across new page, keeping old timestamp %"
-            GST_TIME_FORMAT, granulepos, GST_TIME_ARGS (pad->timestamp));
-      }
-
-      /* push the page */
-      gst_ogg_mux_pad_queue_page (ogg_mux, pad, &page, pad->first_delta);
-      pad->pageno++;
-
-      /* mark next pages as delta */
-      pad->first_delta = TRUE;
-
-      /* use an inner loop here to flush the remaining pages and
-       * mark them as delta frames as well */
-      while (ogg_stream_pageout (&pad->stream, &page) > 0) {
-        /* we have a complete page now, we can push the page
-         * and make sure to pull on a new pad the next time around */
-        GST_LOG_OBJECT (pad->pad,
-            GST_GP_FORMAT " streaming out remaining pages, page %"
-            G_GINT64_FORMAT, ogg_page_granulepos (&page), pad->pageno);
-
-        if (ogg_page_granulepos (&page) == granulepos) {
-          /* the page has taken up the new packet completely, which means
-           * the packet ends the page and we can update the timestamp
-           * before pushing out */
-          GST_LOG_OBJECT (pad->pad,
-              GST_GP_FORMAT
-              " new packet finishes on page, updating timestamp to %"
-              GST_TIME_FORMAT, granulepos, GST_TIME_ARGS (timestamp));
-          pad->timestamp = timestamp;
-        } else {
-          GST_LOG_OBJECT (pad->pad,
-              GST_GP_FORMAT
-              " new packet spans this page, timestamp remains at %"
-              GST_TIME_FORMAT, ogg_page_granulepos (&page),
-              GST_TIME_ARGS (pad->timestamp));
+    if (duration != -1) {
+      pad->duration += duration;
+      /* if page duration exceeds max, flush page */
+      if (pad->duration > ogg_mux->max_page_delay) {
+        while (ogg_stream_flush (&pad->stream, &page)) {
+          gst_ogg_mux_push_page (ogg_mux, &page);
+          /* increment the page number counter */
+          pad->pageno++;
         }
-        gst_ogg_mux_pad_queue_page (ogg_mux, pad, &page, pad->first_delta);
-        /* increment the page number counter */
-        pad->pageno++;
+        pad->duration = 0;
+        return;
       }
+    }
 
-      /* need a new page as well */
-      pad->new_page = TRUE;
-      pad->duration = 0;
-      /* we're done pulling on this pad, make sure to choose a new
+    /* flush out the pages now. The packet we got could end up in
+     * more than one page so we need to flush them all */
+    while (ogg_stream_pageout (&pad->stream, &page) > 0) {
+      /* we have a complete page now, we can push the page 
+       * and make sure to pull on a new pad the next time around */
+      gst_ogg_mux_push_page (ogg_mux, &page);
+      /* increment the page number counter */
+      pad->pageno++;
+      /* we're done pulling on this pad, make sure to choose a new 
        * pad for pulling in the next iteration */
       ogg_mux->pulling = NULL;
-    } else {
-      /* we didn't fill a new page with this new packet */
-      GST_LOG_OBJECT (pad->pad,
-          GST_GP_FORMAT " packet %" G_GINT64_FORMAT
-          " didn't fill a page yet", packet.granulepos, packet.packetno);
+      pad->duration = 0;
     }
-    /* since the new packet now is successfully stored in either the current
-     * page or the last page sent out, all future pages will have at least
-     * the timestamp of this new packet */
-    if (pad->timestamp < timestamp) {
-      GST_LOG_OBJECT (pad->pad,
-          "pad's page timestamp updated to %" GST_TIME_FORMAT,
-          GST_TIME_ARGS (timestamp));
-      pad->timestamp = timestamp;
-    }
-    GST_LOG_OBJECT (pad->pad,
-        GST_GP_FORMAT " packet %" G_GINT64_FORMAT " processed",
-        packet.granulepos, packet.packetno);
   }
 }
 
