@@ -51,7 +51,6 @@ struct _InterleaveInputChannel
 {
   GstPad *sinkpad;
   GstBuffer *buffer;
-  gboolean new_media;
 };
 
 struct _Interleave
@@ -61,8 +60,7 @@ struct _Interleave
   GstPad *srcpad;
   GList *channels;
 
-  gint numpads;                 /* Number of pads on the element */
-  gint numchannels;             /* Actual number of channels */
+  gint numchannels;             /* Number of pads on the element */
   gint channelcount;            /* counter to get safest pad name */
   gboolean is_int;
 
@@ -98,7 +96,7 @@ static GstStaticPadTemplate sink_static_template =
         "signed = (boolean) { true, false };"
         "audio/x-raw-float, "
         "rate = (int) [ 1, MAX ], "
-        "channels = (int) 1,"
+        "channels = (int) 1, "
         "endianness = (int) BYTE_ORDER, "
         "width = (int) 32," "buffer-frames = (int) [ 0, MAX ]")
     );
@@ -127,13 +125,11 @@ static void interleave_init (Interleave * this);
 
 static GstPad *interleave_request_new_pad (GstElement * element,
     GstPadTemplate * temp, const gchar * unused);
-static void interleave_release_pad (GstElement * element, GstPad * pad);
 static void interleave_pad_removed (GstElement * element, GstPad * pad);
 static GstElementStateReturn interleave_change_state (GstElement * element);
 
 static GstCaps *interleave_getcaps (GstPad * pad);
 static GstPadLinkReturn interleave_link (GstPad * pad, const GstCaps * caps);
-static void interleave_unlink (GstPad * pad);
 
 static void interleave_buffered_loop (GstElement * element);
 static void interleave_bytestream_loop (GstElement * element);
@@ -192,7 +188,6 @@ interleave_class_init (InterleaveClass * klass)
   parent_class = g_type_class_ref (GST_TYPE_ELEMENT);
 
   eclass->request_new_pad = interleave_request_new_pad;
-  eclass->release_pad = interleave_release_pad;
   eclass->pad_removed = interleave_pad_removed;
   eclass->change_state = interleave_change_state;
 }
@@ -208,11 +203,7 @@ interleave_init (Interleave * this)
   gst_pad_set_link_function (this->srcpad, interleave_link);
   gst_pad_set_getcaps_function (this->srcpad, interleave_getcaps);
 
-  this->numchannels = 1;        /* Numchannels must start with 1, even if there are
-                                   no channels, as 0 channel audio is invalid
-                                   and it stops people linking the interleave sink
-                                   before the src */
-  this->numpads = 0;
+  this->numchannels = 0;
   this->channelcount = 0;
   this->channels = NULL;
 }
@@ -238,13 +229,10 @@ interleave_request_new_pad (GstElement * element, GstPadTemplate * templ,
   channel->sinkpad = gst_pad_new_from_template (templ, name);
   gst_element_add_pad (GST_ELEMENT (this), channel->sinkpad);
   gst_pad_set_link_function (channel->sinkpad, interleave_link);
-  gst_pad_set_unlink_function (channel->sinkpad, interleave_unlink);
   gst_pad_set_getcaps_function (channel->sinkpad, interleave_getcaps);
 
   this->channels = g_list_append (this->channels, channel);
-  if (this->numpads > 0)
-    this->numchannels++;
-  this->numpads++;
+  this->numchannels++;
   this->channelcount++;
 
   GST_DEBUG ("interleave added pad %s\n", name);
@@ -253,12 +241,6 @@ interleave_request_new_pad (GstElement * element, GstPadTemplate * templ,
   return channel->sinkpad;
 }
 
-static void
-interleave_release_pad (GstElement * element, GstPad * pad)
-{
-  gst_element_remove_pad (element, pad);
-}
-
 static void
 interleave_pad_removed (GstElement * element, GstPad * pad)
 {
@@ -279,9 +261,7 @@ interleave_pad_removed (GstElement * element, GstPad * pad)
 
       p = p->next;
       this->channels = g_list_remove_link (this->channels, p_copy);
-      this->numpads--;
-      if (this->numpads > 0)
-        this->numchannels--;
+      this->numchannels--;
 
       g_list_free (p_copy);
 
@@ -401,8 +381,6 @@ interleave_link (GstPad * pad, const GstCaps * caps)
       return ret;
   }
 
-  g_print ("Interleave has %d channels\n", this->numchannels);
-
   /* it's ok, let's record our data */
   structure = gst_caps_get_structure (caps, 0);
   this->is_int =
@@ -425,45 +403,6 @@ interleave_link (GstPad * pad, const GstCaps * caps)
   return GST_PAD_LINK_OK;
 }
 
-static void
-interleave_unlink (GstPad * pad)
-{
-  Interleave *this;
-  GstCaps *caps;
-
-  this = INTERLEAVE (GST_OBJECT_PARENT (pad));
-
-  if (GST_IS_PAD (this->srcpad)) {
-    gboolean ret;
-
-    caps = gst_pad_get_caps (this->srcpad);
-    gst_caps_set_simple (caps, "channels", G_TYPE_INT, this->numchannels - 1,
-        NULL);
-    ret = gst_pad_try_set_caps (this->srcpad, caps);
-    if (ret == FALSE) {
-      g_print ("TSC failed\n");
-    }
-    gst_pad_renegotiate (this->srcpad);
-    gst_caps_free (caps);
-  }
-}
-
-static gboolean
-all_channels_new_media (GList * channels)
-{
-  GList *p;
-
-  for (p = channels; p; p = p->next) {
-    InterleaveInputChannel *c = INTERLEAVE_CHANNEL (p);
-
-    if (c->new_media == FALSE) {
-      return FALSE;
-    }
-  }
-
-  return TRUE;
-}
-
 static void
 interleave_buffered_loop (GstElement * element)
 {
@@ -490,10 +429,6 @@ interleave_buffered_loop (GstElement * element)
   for (l = this->channels, i = 0; l; l = l->next, i++) {
     channel = INTERLEAVE_CHANNEL (l);
 
-    if (channel->new_media) {
-      continue;
-    }
-
   get_buffer:
     channel->buffer = GST_BUFFER (gst_pad_pull (channel->sinkpad));
 
@@ -510,33 +445,6 @@ interleave_buffered_loop (GstElement * element)
           gst_pad_push (this->srcpad, GST_DATA (e));
           gst_element_set_eos (GST_ELEMENT (this));
           return;
-
-        case GST_EVENT_DISCONTINUOUS:
-          GST_DEBUG ("its a discont");
-
-          if (GST_EVENT_DISCONT_NEW_MEDIA (e)) {
-            channel->new_media = TRUE;
-            if (all_channels_new_media (this->channels)) {
-              GList *t;
-
-              /* Reset new media */
-              for (t = this->channels; t; t = t->next) {
-                InterleaveInputChannel *c = INTERLEAVE_CHANNEL (t);
-
-                c->new_media = FALSE;
-              }
-
-              /* Push event */
-              gst_pad_event_default (channel->sinkpad, e);
-              goto get_buffer;
-            } else {
-              GST_DEBUG ("Not all channels have received a new-media yet");
-              gst_event_unref (e);
-              goto get_buffer;
-            }
-          }
-          break;
-
         default:
           GST_DEBUG ("doing default action for event");
           gst_pad_event_default (channel->sinkpad, e);
@@ -568,7 +476,7 @@ interleave_buffered_loop (GstElement * element)
 
   /* if in the future int gets a buffer-frames property, we should switch here
    * on the type */
-  do_float_interleave ((gfloat **) data_in, this->numpads,
+  do_float_interleave ((gfloat **) data_in, this->numchannels,
       (gfloat *) data_out, to_process_bytes / sizeof (gfloat));
 
   /* clean up and push */
