@@ -29,6 +29,7 @@
 #include <gst/gstelement.h>
 #include <gst/gstversion.h>
 #include <gst/gstinfo.h>
+#include <gst/gstutils.h>
 
 #include <string.h>
 #include <ctype.h>
@@ -326,10 +327,13 @@ mp3_type_find (GstTypeFind * tf, gpointer unused)
 
   while (skipped < GST_MP3_TYPEFIND_TRY_SYNC) {
     if (size <= 0) {
-      data = gst_type_find_peek (tf, skipped, GST_MP3_TYPEFIND_SYNC_SIZE);
+      size = GST_MP3_TYPEFIND_SYNC_SIZE * 2;
+      do {
+        size /= 2;
+        data = gst_type_find_peek (tf, skipped, size);
+      } while (size > 10 && !data);
       if (!data)
         break;
-      size = GST_MP3_TYPEFIND_SYNC_SIZE;
     }
     if (*data == 0xFF) {
       guint8 *head_data = NULL;
@@ -350,7 +354,7 @@ mp3_type_find (GstTypeFind * tf, gpointer unused)
         }
         if (!head_data)
           break;
-        head = GUINT32_FROM_BE (*((guint32 *) head_data));
+        head = GST_READ_UINT32_BE (head_data);
         if (!(length = mp3_type_frame_length_from_header (head, &layer,
                     &channels, &bitrate, &samplerate))) {
           GST_LOG ("%d. header at offset %" G_GUINT64_FORMAT
@@ -498,7 +502,7 @@ mpeg1_parse_header (GstTypeFind * tf, guint64 offset)
         GST_LOG ("couldn't get MPEG pack header bytes");
         return 1;
       }
-      size = GUINT16_FROM_BE (*(guint16 *) data) + 6;
+      size = GST_READ_UINT16_BE (data) + 6;
       offset += 2;
       data = gst_type_find_peek (tf, offset, size - 6);
       if (!data) {
@@ -524,7 +528,7 @@ mpeg1_parse_header (GstTypeFind * tf, guint64 offset)
         GST_LOG ("couldn't get MPEG pack header bytes");
         return 1;
       }
-      size = GUINT16_FROM_BE (*(guint16 *) data) + 6;
+      size = GST_READ_UINT16_BE (data) + 6;
       /* FIXME: we could check PTS/DTS marker bits here... (bit overkill) */
       break;
   }
@@ -729,7 +733,7 @@ qt_type_find (GstTypeFind * tf, gpointer unused)
       tip = GST_TYPE_FIND_MAXIMUM;
       break;
     }
-    offset += GUINT32_FROM_BE (*((guint32 *) data));
+    offset += GST_READ_UINT32_BE (data);
   }
   if (tip > 0) {
     gst_type_find_suggest (tf, tip, QT_CAPS);
@@ -1046,7 +1050,7 @@ matroska_type_find (GstTypeFind * tf, gpointer ununsed)
    * we don't parse the whole header but simply check for the
    * availability of that array of characters inside the header.
    * Not fully fool-proof, but good enough. */
-  for (n = 4 + size; n < 4 + size + total - sizeof (probe_data); n++)
+  for (n = 4 + size; n <= 4 + size + total - sizeof (probe_data); n++)
     if (!memcmp (&data[n], probe_data, sizeof (probe_data))) {
       gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, MATROSKA_CAPS);
       break;
@@ -1168,17 +1172,17 @@ speex_type_find (GstTypeFind * tf, gpointer private)
     data += 32;
 
     /* 4 byte header size >= 80 */
-    if (GINT32_FROM_LE (*((gint32 *) data)) < 80)
+    if (GST_READ_UINT32_LE (data) < 80)
       return;
     data += 4;
 
     /* 4 byte sample rate <= 48000 */
-    if (GINT32_FROM_LE (*((gint32 *) data)) > 48000)
+    if (GST_READ_UINT32_LE (data) > 48000)
       return;
     data += 4;
 
     /* currently there are only 3 speex modes. */
-    if (GINT32_FROM_LE (*((gint32 *) data)) > 3)
+    if (GST_READ_UINT32_LE (data) > 3)
       return;
     data += 12;
 
@@ -1316,7 +1320,7 @@ plugin_init (GstPlugin * plugin)
       au_exts, ".snd", 4, GST_TYPE_FIND_MAXIMUM);
   TYPE_FIND_REGISTER_RIFF (plugin, "video/x-msvideo", GST_RANK_PRIMARY,
       avi_exts, "AVI ");
-  TYPE_FIND_REGISTER_RIFF (plugin, "video/x-cdxa", GST_RANK_SECONDARY,
+  TYPE_FIND_REGISTER_RIFF (plugin, "video/x-cdxa", GST_RANK_PRIMARY,
       cdxa_exts, "CDXA");
   TYPE_FIND_REGISTER_START_WITH (plugin, "audio/x-flac", GST_RANK_PRIMARY,
       flac_exts, "fLaC", 4, GST_TYPE_FIND_MAXIMUM);
@@ -1336,7 +1340,7 @@ plugin_init (GstPlugin * plugin)
       ogg_exts, "OggS", 4, GST_TYPE_FIND_MAXIMUM);
   TYPE_FIND_REGISTER (plugin, "video/mpeg", GST_RANK_SECONDARY,
       mpeg_video_type_find, mpeg_video_exts, MPEG_VIDEO_CAPS, NULL);
-  TYPE_FIND_REGISTER (plugin, "video/mpeg", GST_RANK_MARGINAL,
+  TYPE_FIND_REGISTER (plugin, "video/mpeg-stream", GST_RANK_MARGINAL,
       mpeg_video_stream_type_find, mpeg_video_exts, MPEG_VIDEO_CAPS, NULL);
   TYPE_FIND_REGISTER (plugin, "video/quicktime", GST_RANK_SECONDARY,
       qt_type_find, qt_exts, QT_CAPS, NULL);
@@ -1399,6 +1403,8 @@ plugin_init (GstPlugin * plugin)
       speex_type_find, NULL, SPEEX_CAPS, NULL);
   TYPE_FIND_REGISTER (plugin, "audio/x-m4a", GST_RANK_PRIMARY, m4a_type_find,
       m4a_exts, AAC_CAPS, NULL);
+  TYPE_FIND_REGISTER_START_WITH (plugin, "application/x-executable",
+      GST_RANK_MARGINAL, NULL, "\177ELF", 4, GST_TYPE_FIND_MAXIMUM);
 
   return TRUE;
 }
