@@ -51,13 +51,10 @@ enum
   ARG_0,
   ARG_TYPE,
   ARG_SYNC,
-  ARG_NUM_BUFFERS,
-  ARG_TIMESTAMP_OFFSET,
-  /* FILL ME */
+  ARG_NUM_BUFFERS
+      /* FILL ME */
 };
 
-static GstCaps *capslist = NULL;
-
 static void gst_videotestsrc_base_init (gpointer g_class);
 static void gst_videotestsrc_class_init (GstVideotestsrcClass * klass);
 static void gst_videotestsrc_init (GstVideotestsrc * videotestsrc);
@@ -131,7 +128,7 @@ gst_videotestsrc_pattern_get_type (void)
   static GEnumValue pattern_types[] = {
     {GST_VIDEOTESTSRC_SMPTE, "smpte", "SMPTE 100% color bars"},
     {GST_VIDEOTESTSRC_SNOW, "snow", "Random (television snow)"},
-    {GST_VIDEOTESTSRC_BLACK, "black", "100% Black"},
+    {GST_VIDEOTESTSRC_BLACK, "black", "0% Black"},
     {0, NULL, NULL},
   };
 
@@ -172,10 +169,6 @@ gst_videotestsrc_class_init (GstVideotestsrcClass * klass)
       g_param_spec_int ("num-buffers", "num-buffers",
           "Number of buffers to output before sending EOS", -1, G_MAXINT,
           0, G_PARAM_READWRITE));
-  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_TIMESTAMP_OFFSET,
-      g_param_spec_int64 ("timestamp-offset", "Timestamp offset",
-          "An offset added to timestamps set on buffers (in ns)",
-          G_MININT64, G_MAXINT64, 0, G_PARAM_READWRITE));
 
   parent_class = g_type_class_ref (GST_TYPE_ELEMENT);
 
@@ -256,8 +249,8 @@ gst_videotestsrc_src_link (GstPad * pad, const GstCaps * caps)
 
   videotestsrc->bpp = videotestsrc->fourcc->bitspp;
 
-  GST_DEBUG_OBJECT (videotestsrc, "size %dx%d, %f fps", videotestsrc->width,
-      videotestsrc->height, videotestsrc->rate);
+  GST_DEBUG_OBJECT (videotestsrc, "size %dx%d", videotestsrc->width,
+      videotestsrc->height);
 
   return GST_PAD_LINK_OK;
 }
@@ -289,7 +282,6 @@ gst_videotestsrc_change_state (GstElement * element)
       break;
     case GST_STATE_PAUSED_TO_READY:
       videotestsrc->timestamp_offset = 0;
-      videotestsrc->running_time = 0;
       videotestsrc->n_frames = 0;
       break;
     case GST_STATE_READY_TO_NULL:
@@ -302,29 +294,22 @@ gst_videotestsrc_change_state (GstElement * element)
 static GstCaps *
 gst_videotestsrc_get_capslist (void)
 {
-  return gst_caps_copy (capslist);
-}
+  GstCaps *caps;
+  GstStructure *structure;
+  int i;
 
-static void
-generate_capslist (void)
-{
-  if (!capslist) {
-    GstCaps *caps;
-    GstStructure *structure;
-    int i;
-
-    caps = gst_caps_new_empty ();
-    for (i = 0; i < n_fourccs; i++) {
-      structure = paint_get_structure (fourcc_list + i);
-      gst_structure_set (structure,
-          "width", GST_TYPE_INT_RANGE, 1, G_MAXINT,
-          "height", GST_TYPE_INT_RANGE, 1, G_MAXINT,
-          "framerate", GST_TYPE_DOUBLE_RANGE, 0.0, G_MAXDOUBLE, NULL);
-      gst_caps_append_structure (caps, structure);
-    }
-
-    capslist = caps;
+  caps = gst_caps_new_empty ();
+  for (i = 0; i < n_fourccs; i++) {
+    structure = paint_get_structure (fourcc_list + i);
+    gst_structure_set (structure,
+        "width", GST_TYPE_INT_RANGE, 1, G_MAXINT,
+        "height", GST_TYPE_INT_RANGE, 1, G_MAXINT,
+        "pixel-aspect-ratio", GST_TYPE_FRACTION, 1, 1,
+        "framerate", GST_TYPE_DOUBLE_RANGE, 0.0, G_MAXDOUBLE, NULL);
+    gst_caps_append_structure (caps, structure);
   }
+
+  return caps;
 }
 
 #if 0
@@ -416,7 +401,8 @@ gst_videotestsrc_src_query (GstPad * pad,
     case GST_QUERY_POSITION:
       switch (*format) {
         case GST_FORMAT_TIME:
-          *value = videotestsrc->running_time;
+          *value =
+              videotestsrc->n_frames * GST_SECOND / (double) videotestsrc->rate;
           res = TRUE;
           break;
         case GST_FORMAT_DEFAULT:       /* frames */
@@ -451,23 +437,18 @@ gst_videotestsrc_handle_src_event (GstPad * pad, GstEvent * event)
   gboolean res = TRUE;
   GstVideotestsrc *videotestsrc;
   gint64 new_n_frames;
-  GstClockTime new_running_time;
 
   g_return_val_if_fail (pad != NULL, FALSE);
   g_return_val_if_fail (GST_IS_PAD (pad), FALSE);
 
   videotestsrc = GST_VIDEOTESTSRC (gst_pad_get_parent (pad));
   new_n_frames = videotestsrc->n_frames;
-  new_running_time = videotestsrc->running_time;
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_SEEK:
     {
-      /* since we don't do bookkeeping, we just assign a useful meaning
-       * to the time/frame relation after seeking based on current fps */
       switch (GST_EVENT_SEEK_FORMAT (event)) {
         case GST_FORMAT_TIME:
-          new_running_time = GST_EVENT_SEEK_OFFSET (event);
           new_n_frames =
               GST_EVENT_SEEK_OFFSET (event) * (double) videotestsrc->rate /
               GST_SECOND;
@@ -476,8 +457,6 @@ gst_videotestsrc_handle_src_event (GstPad * pad, GstEvent * event)
           break;
         case GST_FORMAT_DEFAULT:
           new_n_frames = GST_EVENT_SEEK_OFFSET (event);
-          new_running_time = GST_EVENT_SEEK_OFFSET (event) /
-              (double) videotestsrc->rate * GST_SECOND;
           videotestsrc->segment_start_frame = -1;
           videotestsrc->segment_end_frame = -1;
           break;
@@ -521,7 +500,6 @@ gst_videotestsrc_handle_src_event (GstPad * pad, GstEvent * event)
 
   if (videotestsrc->n_frames != new_n_frames) {
     videotestsrc->n_frames = new_n_frames;
-    videotestsrc->running_time = new_running_time;
     videotestsrc->need_discont = TRUE;
   }
 
@@ -587,17 +565,22 @@ gst_videotestsrc_get (GstPad * pad)
   videotestsrc->make_image (videotestsrc, (void *) GST_BUFFER_DATA (buf),
       videotestsrc->width, videotestsrc->height);
 
-  GST_BUFFER_TIMESTAMP (buf) = videotestsrc->timestamp_offset +
-      videotestsrc->running_time;
-  GST_BUFFER_DURATION (buf) = GST_SECOND / (double) videotestsrc->rate;
   if (videotestsrc->sync) {
+    GST_BUFFER_TIMESTAMP (buf) = videotestsrc->timestamp_offset +
+        (videotestsrc->n_frames * GST_SECOND) / (double) videotestsrc->rate;
+    videotestsrc->n_frames++;
+
     /* FIXME this is not correct if we do QoS */
     if (videotestsrc->clock) {
       gst_element_wait (GST_ELEMENT (videotestsrc), GST_BUFFER_TIMESTAMP (buf));
     }
+  } else {
+    GST_BUFFER_TIMESTAMP (buf) = videotestsrc->timestamp_offset +
+        (videotestsrc->n_frames * GST_SECOND) / (double) videotestsrc->rate;
+    videotestsrc->n_frames++;
   }
-  videotestsrc->n_frames++;
-  videotestsrc->running_time += GST_BUFFER_DURATION (buf);
+  GST_BUFFER_DURATION (buf) = GST_SECOND / (double) videotestsrc->rate;
+
   return GST_DATA (buf);
 }
 
@@ -642,9 +625,6 @@ gst_videotestsrc_set_property (GObject * object, guint prop_id,
     case ARG_NUM_BUFFERS:
       videotestsrc->num_buffers = g_value_get_int (value);
       break;
-    case ARG_TIMESTAMP_OFFSET:
-      videotestsrc->timestamp_offset = g_value_get_int64 (value);
-      break;
     default:
       break;
   }
@@ -670,9 +650,6 @@ gst_videotestsrc_get_property (GObject * object, guint prop_id, GValue * value,
     case ARG_NUM_BUFFERS:
       g_value_set_int (value, videotestsrc->num_buffers);
       break;
-    case ARG_TIMESTAMP_OFFSET:
-      g_value_set_int64 (value, videotestsrc->timestamp_offset);
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -687,8 +664,6 @@ plugin_init (GstPlugin * plugin)
   oil_init ();
 #endif
 
-  generate_capslist ();
-
   return gst_element_register (plugin, "videotestsrc", GST_RANK_NONE,
       GST_TYPE_VIDEOTESTSRC);
 }
