@@ -34,7 +34,22 @@
 GST_DEBUG_CATEGORY_STATIC (gst_debug_xvimagesink);
 #define GST_CAT_DEFAULT gst_debug_xvimagesink
 
+typedef struct
+{
+  unsigned long flags;
+  unsigned long functions;
+  unsigned long decorations;
+  long input_mode;
+  unsigned long status;
+}
+MotifWmHints, MwmHints;
+
+#define MWM_HINTS_DECORATIONS   (1L << 1)
+
 static void gst_xvimagesink_buffer_free (GstBuffer * buffer);
+static void gst_xvimagesink_xvimage_destroy (GstXvImageSink * xvimagesink,
+    GstXvImage * xvimage);
+
 
 /* ElementFactory information */
 static GstElementDetails gst_xvimagesink_details =
@@ -50,11 +65,11 @@ static GstStaticPadTemplate gst_xvimagesink_sink_template_factory =
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("video/x-raw-rgb, "
-        "framerate = (double) [ 1.0, 100.0 ], "
+        "framerate = (double) [ 0.0, MAX ], "
         "width = (int) [ 1, MAX ], "
         "height = (int) [ 1, MAX ]; "
         "video/x-raw-yuv, "
-        "framerate = (double) [ 1.0, 100.0 ], "
+        "framerate = (double) [ 0.0, MAX ], "
         "width = (int) [ 1, MAX ], " "height = (int) [ 1, MAX ]")
     );
 
@@ -66,7 +81,8 @@ enum
   ARG_HUE,
   ARG_SATURATION,
   ARG_DISPLAY,
-  ARG_SYNCHRONOUS
+  ARG_SYNCHRONOUS,
+  ARG_PIXEL_ASPECT_RATIO
       /* FILL ME */
 };
 
@@ -97,57 +113,71 @@ gst_xvimagesink_handle_xerror (Display * display, XErrorEvent * xevent)
 static gboolean
 gst_xvimagesink_check_xshm_calls (GstXContext * xcontext)
 {
+#ifndef HAVE_XSHM
+  return FALSE;
+#else
   GstXvImage *xvimage = NULL;
   int (*handler) (Display *, XErrorEvent *);
+  gboolean result = FALSE;
 
   g_return_val_if_fail (xcontext != NULL, FALSE);
 
-#ifdef HAVE_XSHM
   xvimage = g_new0 (GstXvImage, 1);
+  g_return_val_if_fail (xvimage != NULL, FALSE);
 
   /* Setting an error handler to catch failure */
+  error_caught = FALSE;
   handler = XSetErrorHandler (gst_xvimagesink_handle_xerror);
 
-  xvimage->size = (xcontext->bpp / 8);
-
   /* Trying to create a 1x1 picture */
+  GST_DEBUG ("XvShmCreateImage of 1x1");
   xvimage->xvimage = XvShmCreateImage (xcontext->disp, xcontext->xv_port_id,
       xcontext->im_format, NULL, 1, 1, &xvimage->SHMInfo);
+  if (!xvimage->xvimage) {
+    GST_WARNING ("could not XvShmCreateImage a 1x1 image");
+    goto beach;
+  }
+  xvimage->size = xvimage->xvimage->data_size;
 
   xvimage->SHMInfo.shmid = shmget (IPC_PRIVATE, xvimage->size,
       IPC_CREAT | 0777);
+  if (xvimage->SHMInfo.shmid == -1) {
+    GST_WARNING ("could not get shared memory of %d bytes", xvimage->size);
+    goto beach;
+  }
+
   xvimage->SHMInfo.shmaddr = shmat (xvimage->SHMInfo.shmid, 0, 0);
+  if ((int) xvimage->SHMInfo.shmaddr == -1) {
+    GST_WARNING ("Failed to shmat: %s", g_strerror (errno));
+    goto beach;
+  }
+
   xvimage->xvimage->data = xvimage->SHMInfo.shmaddr;
   xvimage->SHMInfo.readOnly = FALSE;
 
-  XShmAttach (xcontext->disp, &xvimage->SHMInfo);
-
-  error_caught = FALSE;
+  if (XShmAttach (xcontext->disp, &xvimage->SHMInfo) == 0) {
+    GST_WARNING ("Failed to XShmAttach");
+    goto beach;
+  }
 
   XSync (xcontext->disp, 0);
 
+  XShmDetach (xcontext->disp, &xvimage->SHMInfo);
+  shmdt (xvimage->SHMInfo.shmaddr);
+  shmctl (xvimage->SHMInfo.shmid, IPC_RMID, 0);
+
+  /* store whether we succeeded in result and reset error_caught */
+  result = !error_caught;
+  error_caught = FALSE;
+
+beach:
   XSetErrorHandler (handler);
-
-  if (error_caught) {           /* Failed, detaching shared memory, destroying image and telling we can't
-                                   use XShm */
-    error_caught = FALSE;
+  if (xvimage->xvimage)
     XFree (xvimage->xvimage);
-    shmdt (xvimage->SHMInfo.shmaddr);
-    shmctl (xvimage->SHMInfo.shmid, IPC_RMID, 0);
-    g_free (xvimage);
-    XSync (xcontext->disp, FALSE);
-    return FALSE;
-  } else {
-    XShmDetach (xcontext->disp, &xvimage->SHMInfo);
-    XFree (xvimage->xvimage);
-    shmdt (xvimage->SHMInfo.shmaddr);
-    shmctl (xvimage->SHMInfo.shmid, IPC_RMID, 0);
-    g_free (xvimage);
-    XSync (xcontext->disp, FALSE);
-  }
+  g_free (xvimage);
+  XSync (xcontext->disp, FALSE);
+  return result;
 #endif /* HAVE_XSHM */
-
-  return TRUE;
 }
 
 /* This function handles GstXvImage creation depending on XShm availability */
@@ -156,66 +186,88 @@ gst_xvimagesink_xvimage_new (GstXvImageSink * xvimagesink,
     gint width, gint height)
 {
   GstXvImage *xvimage = NULL;
+  gboolean succeeded = FALSE;
 
   g_return_val_if_fail (GST_IS_XVIMAGESINK (xvimagesink), NULL);
+  GST_DEBUG_OBJECT (xvimagesink, "creating %dx%d", width, height);
 
   xvimage = g_new0 (GstXvImage, 1);
 
   xvimage->width = width;
   xvimage->height = height;
-  xvimage->data = NULL;
+  xvimage->im_format = xvimagesink->xcontext->im_format;
   xvimage->xvimagesink = xvimagesink;
 
   g_mutex_lock (xvimagesink->x_lock);
 
-  xvimage->size =
-      (xvimagesink->xcontext->bpp / 8) * xvimage->width * xvimage->height;
-
 #ifdef HAVE_XSHM
   if (xvimagesink->xcontext->use_xshm) {
     xvimage->xvimage = XvShmCreateImage (xvimagesink->xcontext->disp,
         xvimagesink->xcontext->xv_port_id,
-        xvimagesink->xcontext->im_format,
-        NULL, xvimage->width, xvimage->height, &xvimage->SHMInfo);
+        xvimage->im_format, NULL,
+        xvimage->width, xvimage->height, &xvimage->SHMInfo);
+    if (!xvimage->xvimage) {
+      GST_ELEMENT_ERROR (xvimagesink, RESOURCE, WRITE, (NULL),
+          ("could not XvShmCreateImage a %dx%d image"));
+      goto beach;
+    }
+
+    /* we have to use the returned data_size for our shm size */
+    xvimage->size = xvimage->xvimage->data_size;
+    GST_DEBUG_OBJECT (xvimagesink, "XShm image size is %d", xvimage->size);
 
     xvimage->SHMInfo.shmid = shmget (IPC_PRIVATE, xvimage->size,
         IPC_CREAT | 0777);
+    if (xvimage->SHMInfo.shmid == -1) {
+      GST_ELEMENT_ERROR (xvimagesink, RESOURCE, WRITE, (NULL),
+          ("could not get shared memory of %d bytes", xvimage->size));
+      goto beach;
+    }
 
     xvimage->SHMInfo.shmaddr = shmat (xvimage->SHMInfo.shmid, 0, 0);
-    xvimage->xvimage->data = xvimage->SHMInfo.shmaddr;
+    if ((int) xvimage->SHMInfo.shmaddr == -1) {
+      GST_ELEMENT_ERROR (xvimagesink, RESOURCE, WRITE, (NULL),
+          ("Failed to shmat: %s", g_strerror (errno)));
+      goto beach;
+    }
 
+    xvimage->xvimage->data = xvimage->SHMInfo.shmaddr;
     xvimage->SHMInfo.readOnly = FALSE;
 
-    XShmAttach (xvimagesink->xcontext->disp, &xvimage->SHMInfo);
+    if (XShmAttach (xvimagesink->xcontext->disp, &xvimage->SHMInfo) == 0) {
+      GST_ELEMENT_ERROR (xvimagesink, RESOURCE, WRITE, (NULL),
+          ("Failed to XShmAttach"));
+      goto beach;
+    }
 
     XSync (xvimagesink->xcontext->disp, FALSE);
-
-    shmctl (xvimage->SHMInfo.shmid, IPC_RMID, 0);
-    xvimage->SHMInfo.shmid = -1;
   } else
 #endif /* HAVE_XSHM */
   {
     xvimage->xvimage = XvCreateImage (xvimagesink->xcontext->disp,
         xvimagesink->xcontext->xv_port_id,
-        xvimagesink->xcontext->im_format,
-        xvimage->data, xvimage->width, xvimage->height);
+        xvimage->im_format, NULL, xvimage->width, xvimage->height);
+    if (!xvimage->xvimage) {
+      GST_ELEMENT_ERROR (xvimagesink, RESOURCE, WRITE, (NULL),
+          ("could not XvCreateImage a %dx%d image"));
+      goto beach;
+    }
 
-    xvimage->data = g_malloc (xvimage->xvimage->data_size);
+    /* we have to use the returned data_size for our image size */
+    xvimage->size = xvimage->xvimage->data_size;
+    xvimage->xvimage->data = g_malloc (xvimage->size);
 
     XSync (xvimagesink->xcontext->disp, FALSE);
   }
+  succeeded = TRUE;
+  g_mutex_unlock (xvimagesink->x_lock);
 
-  if (!xvimage->xvimage) {
-    if (xvimage->data)
-      g_free (xvimage->data);
-
-    g_free (xvimage);
-
+beach:
+  if (!succeeded) {
+    gst_xvimagesink_xvimage_destroy (xvimagesink, xvimage);
     xvimage = NULL;
   }
 
-  g_mutex_unlock (xvimagesink->x_lock);
-
   return xvimage;
 }
 
@@ -235,22 +287,23 @@ gst_xvimagesink_xvimage_destroy (GstXvImageSink * xvimagesink,
 
 #ifdef HAVE_XSHM
   if (xvimagesink->xcontext->use_xshm) {
-    if (xvimage->SHMInfo.shmaddr)
+    if ((int) xvimage->SHMInfo.shmaddr != -1) {
       XShmDetach (xvimagesink->xcontext->disp, &xvimage->SHMInfo);
-
-    if (xvimage->xvimage)
-      XFree (xvimage->xvimage);
-
-    if (xvimage->SHMInfo.shmaddr)
       shmdt (xvimage->SHMInfo.shmaddr);
-
+    }
     if (xvimage->SHMInfo.shmid > 0)
       shmctl (xvimage->SHMInfo.shmid, IPC_RMID, 0);
+    if (xvimage->xvimage)
+      XFree (xvimage->xvimage);
   } else
 #endif /* HAVE_XSHM */
   {
-    if (xvimage->xvimage)
+    if (xvimage->xvimage) {
+      if (xvimage->xvimage->data) {
+        g_free (xvimage->xvimage->data);
+      }
       XFree (xvimage->xvimage);
+    }
   }
 
   XSync (xvimagesink->xcontext->disp, FALSE);
@@ -276,6 +329,10 @@ gst_xvimagesink_xvimage_put (GstXvImageSink * xvimagesink, GstXvImage * xvimage)
   /* We scale to the window's geometry */
 #ifdef HAVE_XSHM
   if (xvimagesink->xcontext->use_xshm) {
+    GST_LOG_OBJECT (xvimagesink,
+        "XvShmPutImage with image %dx%d and window %dx%d",
+        xvimage->width, xvimage->height,
+        xvimagesink->xwindow->width, xvimagesink->xwindow->height);
     XvShmPutImage (xvimagesink->xcontext->disp,
         xvimagesink->xcontext->xv_port_id,
         xvimagesink->xwindow->win,
@@ -298,7 +355,44 @@ gst_xvimagesink_xvimage_put (GstXvImageSink * xvimagesink, GstXvImage * xvimage)
   g_mutex_unlock (xvimagesink->x_lock);
 }
 
-/* This function handles a GstXWindow creation */
+static gboolean
+gst_xvimagesink_xwindow_decorate (GstXvImageSink * xvimagesink,
+    GstXWindow * window)
+{
+  Atom hints_atom = None;
+  MotifWmHints *hints;
+
+  g_return_val_if_fail (GST_IS_XVIMAGESINK (xvimagesink), FALSE);
+  g_return_val_if_fail (window != NULL, FALSE);
+
+  g_mutex_lock (xvimagesink->x_lock);
+
+  hints_atom = XInternAtom (xvimagesink->xcontext->disp, "_MOTIF_WM_HINTS", 1);
+  if (hints_atom == None) {
+    g_mutex_unlock (xvimagesink->x_lock);
+    return FALSE;
+  }
+
+  hints = g_malloc0 (sizeof (MotifWmHints));
+
+  hints->flags |= MWM_HINTS_DECORATIONS;
+  hints->decorations = 1 << 0;
+
+  XChangeProperty (xvimagesink->xcontext->disp, window->win,
+      hints_atom, hints_atom, 32, PropModeReplace,
+      (guchar *) hints, sizeof (MotifWmHints) / sizeof (long));
+
+  XSync (xvimagesink->xcontext->disp, FALSE);
+
+  g_mutex_unlock (xvimagesink->x_lock);
+
+  g_free (hints);
+
+  return TRUE;
+}
+
+/* This function handles a GstXWindow creation
+ * The width and height are the actual pixel size on the display */
 static GstXWindow *
 gst_xvimagesink_xwindow_new (GstXvImageSink * xvimagesink,
     gint width, gint height)
@@ -334,6 +428,8 @@ gst_xvimagesink_xwindow_new (GstXvImageSink * xvimagesink,
 
   g_mutex_unlock (xvimagesink->x_lock);
 
+  gst_xvimagesink_xwindow_decorate (xvimagesink, xwindow);
+
   gst_x_overlay_got_xwindow_id (GST_X_OVERLAY (xvimagesink), xwindow->win);
 
   return xwindow;
@@ -364,7 +460,8 @@ gst_xvimagesink_xwindow_destroy (GstXvImageSink * xvimagesink,
   g_free (xwindow);
 }
 
-/* This function resizes a GstXWindow */
+/* This function resizes a GstXWindow.
+ * The width and height are the actual pixel size on the display. */
 static void
 gst_xvimagesink_xwindow_resize (GstXvImageSink * xvimagesink,
     GstXWindow * xwindow, guint width, guint height)
@@ -570,7 +667,7 @@ gst_xvimagesink_handle_xevents (GstXvImageSink * xvimagesink, GstPad * pad)
 
 /* This function generates a caps with all supported format by the first
    Xv grabable port we find. We store each one of the supported formats in a
-   format list and append the format to a newly created caps that we return 
+   format list and append the format to a newly created caps that we return
    If this function does not return NULL because of an error, it also grabs
    the port via XvGrabPort */
 static GstCaps *
@@ -671,9 +768,9 @@ gst_xvimagesink_get_xv_support (GstXvImageSink * xvimagesink,
             "red_mask", G_TYPE_INT, formats[i].blue_mask,
             "width", GST_TYPE_INT_RANGE, 1, G_MAXINT,
             "height", GST_TYPE_INT_RANGE, 1, G_MAXINT,
-            "framerate", GST_TYPE_DOUBLE_RANGE, 1.0, 100.0, NULL);
+            "framerate", GST_TYPE_DOUBLE_RANGE, 0.0, G_MAXDOUBLE, NULL);
 
-        /* For RGB caps we store them and the image 
+        /* For RGB caps we store them and the image
            format so that we can get back the format
            when sinkconnect will give us a caps without
            format property */
@@ -695,7 +792,7 @@ gst_xvimagesink_get_xv_support (GstXvImageSink * xvimagesink,
             "format", GST_TYPE_FOURCC, formats[i].id,
             "width", GST_TYPE_INT_RANGE, 1, G_MAXINT,
             "height", GST_TYPE_INT_RANGE, 1, G_MAXINT,
-            "framerate", GST_TYPE_DOUBLE_RANGE, 1.0, 100.0, NULL);
+            "framerate", GST_TYPE_DOUBLE_RANGE, 0.0, G_MAXDOUBLE, NULL);
         break;
       default:
         g_assert_not_reached ();
@@ -721,9 +818,61 @@ gst_xvimagesink_get_xv_support (GstXvImageSink * xvimagesink,
   return caps;
 }
 
-/* This function get the X Display and global infos about it. Everything is
+/* This function calculates the pixel aspect ratio based on the properties
+ * in the xcontext structure and stores it there. */
+static void
+gst_xvimagesink_calculate_pixel_aspect_ratio (GstXContext * xcontext)
+{
+  gint par[][2] = {
+    { 1, 1},            /* regular screen */
+    { 16, 15 },         /* PAL TV */
+    { 11, 10 },         /* 525 line Rec.601 video */
+    { 54, 59 }          /* 625 line Rec.601 video */
+  };
+  gint i;
+  gint index;
+  gdouble ratio;
+  gdouble delta;
+
+#define DELTA(idx) (ABS (ratio - ((gdouble) par[idx][0] / par[idx][1])))
+
+  /* first calculate the "real" ratio based on the X values;
+   * which is the "physical" w/h divided by the w/h in pixels of the display */
+  ratio = xcontext->widthmm * xcontext->height
+      / (xcontext->heightmm * xcontext->width);
+  /* DirectFB's X in 720x576 reports the physical dimensions wrong, so
+   * override here */
+  if (xcontext->width == 720 && xcontext->height == 576) {
+    ratio = 4.0 * 576 / (3.0 * 720);
+  }
+  GST_DEBUG ("calculated pixel aspect ratio: %f", ratio);
+  /* now find the one from par[][2] with the lowest delta to the real one */
+  delta = DELTA (0);
+  index = 0;
+
+  for (i = 1; i < sizeof (par) / (sizeof (gint) * 2); ++i) {
+    gdouble this_delta = DELTA (i);
+    if (this_delta < delta) {
+      index = i;
+      delta = this_delta;
+    }
+  }
+
+  GST_DEBUG ("Decided on index %d (%d/%d)", index,
+      par[index][0], par[index][1]);
+
+  g_free (xcontext->par);
+  xcontext->par = g_new0 (GValue, 1);
+  g_value_init (xcontext->par, GST_TYPE_FRACTION);
+  gst_value_set_fraction (xcontext->par, par[index][0], par[index][1]);
+  GST_DEBUG ("set xcontext PAR to %d/%d",
+      gst_value_get_fraction_numerator (xcontext->par),
+      gst_value_get_fraction_denominator (xcontext->par));
+}
+
+/* This function gets the X Display and global info about it. Everything is
    stored in our object and will be cleaned when the object is disposed. Note
-   here that caps for supported format are generated without any window or 
+   here that caps for supported format are generated without any window or
    image creation */
 static GstXContext *
 gst_xvimagesink_xcontext_get (GstXvImageSink * xvimagesink)
@@ -761,6 +910,12 @@ gst_xvimagesink_xcontext_get (GstXvImageSink * xvimagesink)
   xcontext->black = XBlackPixel (xcontext->disp, xcontext->screen_num);
   xcontext->depth = DefaultDepthOfScreen (xcontext->screen);
 
+  xcontext->width = DisplayWidth (xcontext->disp, xcontext->screen_num);
+  xcontext->height = DisplayHeight (xcontext->disp, xcontext->screen_num);
+  xcontext->widthmm = DisplayWidthMM (xcontext->disp, xcontext->screen_num);
+  xcontext->heightmm = DisplayHeightMM (xcontext->disp, xcontext->screen_num);
+
+  gst_xvimagesink_calculate_pixel_aspect_ratio (xcontext);
   /* We get supported pixmap formats at supported depth */
   px_formats = XListPixmapFormats (xcontext->disp, &nb_formats);
 
@@ -912,6 +1067,7 @@ gst_xvimagesink_xcontext_clear (GstXvImageSink * xvimagesink)
     g_list_free (xvimagesink->xcontext->channels_list);
 
   gst_caps_free (xvimagesink->xcontext->caps);
+  g_free (xvimagesink->xcontext->par);
 
   g_mutex_lock (xvimagesink->x_lock);
 
@@ -1008,13 +1164,7 @@ gst_xvimagesink_getcaps (GstPad * pad)
   if (xvimagesink->xcontext)
     return gst_caps_copy (xvimagesink->xcontext->caps);
 
-  return gst_caps_from_string ("video/x-raw-rgb, "
-      "framerate = (double) [ 1.0, 100.0 ], "
-      "width = (int) [ 0, MAX ], "
-      "height = (int) [ 0, MAX ]; "
-      "video/x-raw-yuv, "
-      "framerate = (double) [ 0, MAX ], "
-      "width = (int) [ 0, MAX ], " "height = (int) [ 0, MAX ]");
+  return gst_caps_copy (gst_pad_get_pad_template_caps (pad));
 }
 
 static GstPadLinkReturn
@@ -1024,6 +1174,12 @@ gst_xvimagesink_sink_link (GstPad * pad, const GstCaps * caps)
   GstStructure *structure;
   gint im_format = 0;
   gboolean ret;
+  gint video_width, video_height;
+  gint video_par_n, video_par_d;	/* video's PAR */
+  gint display_par_n, display_par_d;    /* display's PAR */
+  GValue display_ratio = { 0, };	/* display w/h ratio */
+  const GValue *caps_par;
+  gint num, den;
 
   xvimagesink = GST_XVIMAGESINK (gst_pad_get_parent (pad));
 
@@ -1033,14 +1189,16 @@ gst_xvimagesink_sink_link (GstPad * pad, const GstCaps * caps)
 
   structure = gst_caps_get_structure (caps, 0);
   ret = gst_structure_get_int (structure, "width",
-      &(GST_VIDEOSINK_WIDTH (xvimagesink)));
+      &video_width);
   ret &= gst_structure_get_int (structure, "height",
-      &(GST_VIDEOSINK_HEIGHT (xvimagesink)));
+      &video_height);
   ret &= gst_structure_get_double (structure, "framerate",
       &xvimagesink->framerate);
   if (!ret)
     return GST_PAD_LINK_REFUSED;
 
+  xvimagesink->video_width = video_width;
+  xvimagesink->video_height = video_height;
   if (!gst_structure_get_fourcc (structure, "format", &im_format)) {
     im_format =
         gst_xvimagesink_get_fourcc_from_caps (xvimagesink,
@@ -1050,13 +1208,63 @@ gst_xvimagesink_sink_link (GstPad * pad, const GstCaps * caps)
     return GST_PAD_LINK_REFUSED;
   }
 
-  xvimagesink->pixel_width = 1;
-  gst_structure_get_int (structure, "pixel_width", &xvimagesink->pixel_width);
+  /* get aspect ratio from caps if it's present, and
+   * convert video width and height to a display width and height
+   * using wd / hd = wv / hv * PARv / PARd
+   * the ratio wd / hd will be stored in display_ratio */
+  g_value_init (&display_ratio, GST_TYPE_FRACTION);
 
-  xvimagesink->pixel_height = 1;
-  gst_structure_get_int (structure, "pixel_height", &xvimagesink->pixel_height);
+  /* get video's PAR */
+  caps_par = gst_structure_get_value (structure, "pixel-aspect-ratio");
+  if (caps_par) {
+    video_par_n = gst_value_get_fraction_numerator (caps_par);
+    video_par_d = gst_value_get_fraction_denominator (caps_par);
+  } else {
+    video_par_n = 1;
+    video_par_d = 1;
+  }
+  /* get display's PAR */
+  if (xvimagesink->par) {
+    display_par_n = gst_value_get_fraction_numerator (xvimagesink->par);
+    display_par_d = gst_value_get_fraction_denominator (xvimagesink->par);
+  } else {
+    display_par_n = 1;
+    display_par_d = 1;
+  }
 
-  /* Creating our window and our image */
+  gst_value_set_fraction (&display_ratio,
+      video_width * video_par_n * display_par_d,
+      video_height * video_par_d * display_par_n);
+
+  num = gst_value_get_fraction_numerator (&display_ratio);
+  den = gst_value_get_fraction_denominator (&display_ratio);
+  GST_DEBUG_OBJECT (xvimagesink,
+      "video width/height: %dx%d, calculated display ratio: %d/%d",
+      video_width, video_height, num, den);
+
+  /* now find a width x height that respects this display ratio.
+   * prefer those that have one of w/h the same as the incoming video
+   * using wd / hd = num / den */
+
+  /* start with same height, because of interlaced video */
+  /* check hd / den is an integer scale factor, and scale wd with the PAR */
+  if (video_height % den == 0) {
+    GST_DEBUG_OBJECT (xvimagesink, "keeping video height");
+    GST_VIDEOSINK_WIDTH (xvimagesink) = video_height * num / den;
+    GST_VIDEOSINK_HEIGHT (xvimagesink) = video_height;
+  } else if (video_width % num == 0) {
+    GST_DEBUG_OBJECT (xvimagesink, "keeping video width");
+    GST_VIDEOSINK_WIDTH (xvimagesink) = video_width;
+    GST_VIDEOSINK_HEIGHT (xvimagesink) = video_width * den / num;
+  } else {
+    GST_DEBUG_OBJECT (xvimagesink, "approximating while keeping video height");
+    GST_VIDEOSINK_WIDTH (xvimagesink) = video_height * num / den;
+    GST_VIDEOSINK_HEIGHT (xvimagesink) = video_height;
+  }
+  GST_DEBUG_OBJECT (xvimagesink, "scaling to %dx%d",
+      GST_VIDEOSINK_WIDTH (xvimagesink), GST_VIDEOSINK_HEIGHT (xvimagesink));
+
+  /* Creating our window and our image with the display size in pixels */
   g_assert (GST_VIDEOSINK_WIDTH (xvimagesink) > 0);
   g_assert (GST_VIDEOSINK_HEIGHT (xvimagesink) > 0);
   if (!xvimagesink->xwindow)
@@ -1069,29 +1277,23 @@ gst_xvimagesink_sink_link (GstPad * pad, const GstCaps * caps)
           GST_VIDEOSINK_HEIGHT (xvimagesink));
   }
 
-  /* We renew our xvimage only if size or format changed */
+  /* We renew our xvimage only if size or format changed;
+   * the xvimage is the same size as the video pixel size */
   if ((xvimagesink->xvimage) &&
-      ((xvimagesink->xcontext->im_format != im_format) ||
-          (GST_VIDEOSINK_WIDTH (xvimagesink) != xvimagesink->xvimage->width) ||
-          (GST_VIDEOSINK_HEIGHT (xvimagesink) !=
-              xvimagesink->xvimage->height))) {
+      ((im_format != xvimagesink->xvimage->im_format) ||
+          (video_width != xvimagesink->xvimage->width) ||
+          (video_height != xvimagesink->xvimage->height))) {
     GST_DEBUG_OBJECT (xvimagesink,
         "old format " GST_FOURCC_FORMAT ", new format " GST_FOURCC_FORMAT,
         GST_FOURCC_ARGS (xvimagesink->xcontext->im_format),
         GST_FOURCC_ARGS (im_format));
     GST_DEBUG_OBJECT (xvimagesink, "renewing xvimage");
     gst_xvimagesink_xvimage_destroy (xvimagesink, xvimagesink->xvimage);
-
-    xvimagesink->xcontext->im_format = im_format;
-    xvimagesink->xvimage = gst_xvimagesink_xvimage_new (xvimagesink,
-        GST_VIDEOSINK_WIDTH (xvimagesink), GST_VIDEOSINK_HEIGHT (xvimagesink));
-  } else if (!xvimagesink->xvimage) {
-    /* If no xvimage, creating one */
-    xvimagesink->xcontext->im_format = im_format;
-    xvimagesink->xvimage = gst_xvimagesink_xvimage_new (xvimagesink,
-        GST_VIDEOSINK_WIDTH (xvimagesink), GST_VIDEOSINK_HEIGHT (xvimagesink));
+    xvimagesink->xvimage = NULL;
   }
 
+  xvimagesink->xcontext->im_format = im_format;
+
   gst_x_overlay_got_desired_size (GST_X_OVERLAY (xvimagesink),
       GST_VIDEOSINK_WIDTH (xvimagesink), GST_VIDEOSINK_HEIGHT (xvimagesink));
 
@@ -1111,6 +1313,16 @@ gst_xvimagesink_change_state (GstElement * element)
       if (!xvimagesink->xcontext &&
           !(xvimagesink->xcontext = gst_xvimagesink_xcontext_get (xvimagesink)))
         return GST_STATE_FAILURE;
+      /* update object's par with calculated one if not set yet */
+      if (!xvimagesink->par) {
+        xvimagesink->par = g_new0 (GValue, 1);
+        gst_value_init_and_copy (xvimagesink->par, xvimagesink->xcontext->par);
+        GST_DEBUG_OBJECT (xvimagesink, "set calculated PAR on object's PAR");
+      }
+      /* call XSynchronize with the current value of synchronous */
+      GST_DEBUG_OBJECT (xvimagesink, "XSynchronize called with %s",
+          xvimagesink->synchronous ? "TRUE" : "FALSE");
+      XSynchronize (xvimagesink->xcontext->disp, xvimagesink->synchronous);
       gst_xvimagesink_update_colorbalance (xvimagesink);
       break;
     case GST_STATE_READY_TO_PAUSED:
@@ -1175,7 +1387,7 @@ gst_xvimagesink_chain (GstPad * pad, GstData * data)
   if (GST_BUFFER_TIMESTAMP_IS_VALID (buf)) {
     xvimagesink->time = GST_BUFFER_TIMESTAMP (buf);
   }
-  GST_DEBUG ("clock wait: %" GST_TIME_FORMAT,
+  GST_LOG_OBJECT (xvimagesink, "clock wait: %" GST_TIME_FORMAT,
       GST_TIME_ARGS (xvimagesink->time));
 
   if (GST_VIDEOSINK_CLOCK (xvimagesink)) {
@@ -1186,20 +1398,26 @@ gst_xvimagesink_chain (GstPad * pad, GstData * data)
      put the ximage which is in the PRIVATE pointer */
   if (GST_BUFFER_FREE_DATA_FUNC (buf) == gst_xvimagesink_buffer_free) {
     gst_xvimagesink_xvimage_put (xvimagesink, GST_BUFFER_PRIVATE (buf));
-  } else {                      /* Else we have to copy the data into our private image, */
+  } else {
+    /* Else we have to copy the data into our private image, */
     /* if we have one... */
-    if (xvimagesink->xvimage) {
-      memcpy (xvimagesink->xvimage->xvimage->data,
-          GST_BUFFER_DATA (buf),
-          MIN (GST_BUFFER_SIZE (buf), xvimagesink->xvimage->size));
-      gst_xvimagesink_xvimage_put (xvimagesink, xvimagesink->xvimage);
-    } else {                    /* No image available. Something went wrong during capsnego ! */
-
-      gst_buffer_unref (buf);
-      GST_ELEMENT_ERROR (xvimagesink, CORE, NEGOTIATION, (NULL),
-          ("no format defined before chain function"));
-      return;
+    if (!xvimagesink->xvimage) {
+      GST_DEBUG_OBJECT (xvimagesink, "creating our xvimage");
+      xvimagesink->xvimage = gst_xvimagesink_xvimage_new (xvimagesink,
+          xvimagesink->video_width, xvimagesink->video_height);
+      if (!xvimagesink->xvimage) {
+        /* No image available. That's very bad ! */
+        gst_buffer_unref (buf);
+        GST_ELEMENT_ERROR (xvimagesink, CORE, NEGOTIATION, (NULL),
+            ("Failed creating an XvImage in xvimagesink chain function."));
+        return;
+      }
     }
+
+    memcpy (xvimagesink->xvimage->xvimage->data,
+        GST_BUFFER_DATA (buf),
+        MIN (GST_BUFFER_SIZE (buf), xvimagesink->xvimage->size));
+    gst_xvimagesink_xvimage_put (xvimagesink, xvimagesink->xvimage);
   }
 
   /* set correct time for next buffer */
@@ -1229,8 +1447,8 @@ gst_xvimagesink_buffer_free (GstBuffer * buffer)
   if ((xvimage->width != GST_VIDEOSINK_WIDTH (xvimagesink)) ||
       (xvimage->height != GST_VIDEOSINK_HEIGHT (xvimagesink)))
     gst_xvimagesink_xvimage_destroy (xvimagesink, xvimage);
-  else {                        /* In that case we can reuse the image and add it to our image pool. */
-
+  else {
+    /* In that case we can reuse the image and add it to our image pool. */
     g_mutex_lock (xvimagesink->pool_lock);
     xvimagesink->image_pool = g_slist_prepend (xvimagesink->image_pool,
         xvimage);
@@ -1250,7 +1468,7 @@ gst_xvimagesink_buffer_alloc (GstPad * pad, guint64 offset, guint size)
 
   g_mutex_lock (xvimagesink->pool_lock);
 
-  /* Walking through the pool cleaning unsuable images and searching for a
+  /* Walking through the pool cleaning unusable images and searching for a
      suitable one */
   while (not_found && xvimagesink->image_pool) {
     xvimage = xvimagesink->image_pool->data;
@@ -1260,11 +1478,15 @@ gst_xvimagesink_buffer_alloc (GstPad * pad, guint64 offset, guint size)
       xvimagesink->image_pool = g_slist_delete_link (xvimagesink->image_pool,
           xvimagesink->image_pool);
 
-      if ((xvimage->width != GST_VIDEOSINK_WIDTH (xvimagesink)) || (xvimage->height != GST_VIDEOSINK_HEIGHT (xvimagesink))) {   /* This image is unusable. Destroying... */
+      /* We check for geometry or image format changes */
+      if ((xvimage->width != GST_VIDEOSINK_WIDTH (xvimagesink)) ||
+          (xvimage->height != GST_VIDEOSINK_HEIGHT (xvimagesink)) ||
+          (xvimage->im_format != xvimagesink->xcontext->im_format)) {
+        /* This image is unusable. Destroying... */
         gst_xvimagesink_xvimage_destroy (xvimagesink, xvimage);
         xvimage = NULL;
-      } else {                  /* We found a suitable image */
-
+      } else {
+        /* We found a suitable image */
         break;
       }
     }
@@ -1272,9 +1494,11 @@ gst_xvimagesink_buffer_alloc (GstPad * pad, guint64 offset, guint size)
 
   g_mutex_unlock (xvimagesink->pool_lock);
 
-  if (!xvimage) {               /* We found no suitable image in the pool. Creating... */
+  if (!xvimage) {
+    /* We found no suitable image in the pool. Creating... */
+    GST_DEBUG_OBJECT (xvimagesink, "no usable image in pool, creating xvimage");
     xvimage = gst_xvimagesink_xvimage_new (xvimagesink,
-        GST_VIDEOSINK_WIDTH (xvimagesink), GST_VIDEOSINK_HEIGHT (xvimagesink));
+        xvimagesink->video_width, xvimagesink->video_height);
   }
 
   if (xvimage) {
@@ -1406,13 +1630,6 @@ gst_xvimagesink_set_xwindow_id (GstXOverlay * overlay, XID xwindow_id)
     g_mutex_unlock (xvimagesink->x_lock);
   }
 
-  /* Recreating our xvimage */
-  if (!xvimagesink->xvimage &&
-      GST_VIDEOSINK_WIDTH (xvimagesink) && GST_VIDEOSINK_HEIGHT (xvimagesink)) {
-    xvimagesink->xvimage = gst_xvimagesink_xvimage_new (xvimagesink,
-        GST_VIDEOSINK_WIDTH (xvimagesink), GST_VIDEOSINK_HEIGHT (xvimagesink));
-  }
-
   if (xwindow)
     xvimagesink->xwindow = xwindow;
 }
@@ -1584,8 +1801,22 @@ gst_xvimagesink_set_property (GObject * object, guint prop_id,
       xvimagesink->synchronous = g_value_get_boolean (value);
       if (xvimagesink->xcontext) {
         XSynchronize (xvimagesink->xcontext->disp, xvimagesink->synchronous);
+        GST_DEBUG_OBJECT (xvimagesink, "XSynchronize called with %s",
+            xvimagesink->synchronous ? "TRUE" : "FALSE");
       }
       break;
+    case ARG_PIXEL_ASPECT_RATIO:
+      g_free (xvimagesink->par);
+      xvimagesink->par = g_new0 (GValue, 1);
+      g_value_init (xvimagesink->par, GST_TYPE_FRACTION);
+      if (!g_value_transform (value, xvimagesink->par)) {
+        g_warning ("Could not transform string to aspect ratio");
+        gst_value_set_fraction (xvimagesink->par, 1, 1);
+      }
+      GST_DEBUG_OBJECT (xvimagesink, "set PAR to %d/%d",
+          gst_value_get_fraction_numerator (xvimagesink->par),
+          gst_value_get_fraction_denominator (xvimagesink->par));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -1621,6 +1852,10 @@ gst_xvimagesink_get_property (GObject * object, guint prop_id,
     case ARG_SYNCHRONOUS:
       g_value_set_boolean (value, xvimagesink->synchronous);
       break;
+    case ARG_PIXEL_ASPECT_RATIO:
+      if (xvimagesink->par)
+        g_value_transform (xvimagesink->par, value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -1683,12 +1918,11 @@ gst_xvimagesink_init (GstXvImageSink * xvimagesink)
 
   xvimagesink->x_lock = g_mutex_new ();
 
-  xvimagesink->pixel_width = xvimagesink->pixel_height = 1;
-
   xvimagesink->image_pool = NULL;
   xvimagesink->pool_lock = g_mutex_new ();
 
   xvimagesink->synchronous = FALSE;
+  xvimagesink->par = NULL;
 
   GST_FLAG_SET (xvimagesink, GST_ELEMENT_THREAD_SUGGESTED);
   GST_FLAG_SET (xvimagesink, GST_ELEMENT_EVENT_AWARE);
@@ -1736,6 +1970,10 @@ gst_xvimagesink_class_init (GstXvImageSinkClass * klass)
           "When enabled, runs "
           "the X display in synchronous mode. (used only for debugging)", FALSE,
           G_PARAM_READWRITE));
+  g_object_class_install_property (gobject_class, ARG_PIXEL_ASPECT_RATIO,
+      g_param_spec_string ("pixel-aspect-ratio", "Pixel Aspect Ratio",
+          "The pixel aspect ratio of the device", "1/1",
+          G_PARAM_READWRITE));
 
   gobject_class->finalize = gst_xvimagesink_finalize;
   gobject_class->set_property = gst_xvimagesink_set_property;
